#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iam_group
version_added: 1.0.0
version_added_collection: community.aws
short_description: Manage AWS IAM groups
description:
  - Manage AWS IAM groups.
author:
  - Nick Aslanidis (@naslanidis)
  - Maksym Postument (@infectsoldier)
options:
  name:
    description:
      - The name of the group.
      - >-
        Note: Group names are unique within an account.  Paths (O(path)) do B(not) affect
        the uniqueness requirements of O(name).  For example it is not permitted to have both
        C(/Path1/MyGroup) and C(/Path2/MyGroup) in the same account.
      - The alias O(group_name) was added in release 7.2.0.
    required: true
    aliases: ['group_name']
    type: str
  path:
    description:
      - The group path.
      - For more information about IAM paths, see the AWS IAM identifiers documentation
        U(https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_identifiers.html).
    aliases: ['prefix', 'path_prefix']
    version_added: 7.1.0
    type: str
  managed_policies:
    description:
      - A list of managed policy ARNs or friendly names to attach to the role.
      - If known, it is recommended to use ARNs rather than friendly names to avoid additional
        lookups.
      - To embed an inline policy, use M(amazon.aws.iam_policy).
    required: false
    type: list
    elements: str
    default: []
    aliases: ['managed_policy']
  users:
    description:
      - A list of existing users to add as members of the group.
    required: false
    type: list
    elements: str
    default: []
  state:
    description:
      - Create or remove the IAM group.
    required: true
    choices: [ 'present', 'absent' ]
    type: str
  purge_policies:
    description:
      - When O(purge_policies=true) any managed policies not listed in O(managed_policies) will be detatched.
    required: false
    default: false
    type: bool
    aliases: ['purge_policy', 'purge_managed_policies']
  purge_users:
    description:
      - When O(purge_users=true) users which are not included in I(users) will be detached.
    required: false
    default: false
    type: bool
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Create a group
  amazon.aws.iam_group:
    name: testgroup1
    state: present

- name: Create a group and attach a managed policy using its ARN
  amazon.aws.iam_group:
    name: testgroup1
    managed_policies:
      - arn:aws:iam::aws:policy/AmazonSNSFullAccess
    state: present

- name: Create a group with users as members and attach a managed policy using its ARN
  amazon.aws.iam_group:
    name: testgroup1
    managed_policies:
      - arn:aws:iam::aws:policy/AmazonSNSFullAccess
    users:
      - test_user1
      - test_user2
    state: present

- name: Remove all managed policies from an existing group with an empty list
  amazon.aws.iam_group:
    name: testgroup1
    state: present
    purge_policies: true

- name: Remove all group members from an existing group
  amazon.aws.iam_group:
    name: testgroup1
    managed_policies:
      - arn:aws:iam::aws:policy/AmazonSNSFullAccess
    purge_users: true
    state: present

- name: Delete the group
  amazon.aws.iam_group:
    name: testgroup1
    state: absent
"""

RETURN = r"""
iam_group:
    description: Dictionary containing all the group information including group membership.
    returned: success
    type: complex
    contains:
        group:
            description: Dictionary containing all the group information.
            returned: success
            type: complex
            contains:
                arn:
                    description: The Amazon Resource Name (ARN) specifying the group.
                    type: str
                    sample: "arn:aws:iam::1234567890:group/testgroup1"
                create_date:
                    description: The date and time, in ISO 8601 date-time format, when the group was created.
                    type: str
                    sample: "2017-02-08T04:36:28+00:00"
                group_id:
                    description: The stable and unique string identifying the group.
                    type: str
                    sample: AGPA12345EXAMPLE54321
                group_name:
                    description: The friendly name that identifies the group.
                    type: str
                    sample: testgroup1
                path:
                    description: The path to the group.
                    type: str
                    sample: /
        users:
            description: List containing all the group members.
            returned: success
            type: complex
            contains:
                arn:
                    description: The Amazon Resource Name (ARN) specifying the user.
                    type: str
                    sample: "arn:aws:iam::1234567890:user/test_user1"
                create_date:
                    description: The date and time, in ISO 8601 date-time format, when the user was created.
                    type: str
                    sample: "2017-02-08T04:36:28+00:00"
                user_id:
                    description: The stable and unique string identifying the user.
                    type: str
                    sample: AIDA12345EXAMPLE54321
                user_name:
                    description: The friendly name that identifies the user.
                    type: str
                    sample: testgroup1
                path:
                    description: The path to the user.
                    type: str
                    sample: /
        attached_policies:
            version_added: 7.1.0
            description:
                - List containing basic information about managed policies attached to the group.
            returned: success
            type: complex
            contains:
                policy_arn:
                    description: The Amazon Resource Name (ARN) specifying the managed policy.
                    type: str
                    sample: "arn:aws:iam::123456789012:policy/test_policy"
                policy_name:
                    description: The friendly name that identifies the policy.
                    type: str
                    sample: test_policy
"""

from ansible_collections.amazon.aws.plugins.module_utils.iam import AnsibleIAMError
from ansible_collections.amazon.aws.plugins.module_utils.iam import IAMErrorHandler
from ansible_collections.amazon.aws.plugins.module_utils.iam import convert_managed_policy_names_to_arns
from ansible_collections.amazon.aws.plugins.module_utils.iam import get_iam_group
from ansible_collections.amazon.aws.plugins.module_utils.iam import normalize_iam_group
from ansible_collections.amazon.aws.plugins.module_utils.iam import validate_iam_identifiers
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


@IAMErrorHandler.common_error_handler("update group path")
def ensure_path(connection, module, group_info, path):
    if path is None:
        return False

    if group_info["Group"].get("Path") == path:
        return False

    if module.check_mode:
        return True

    connection.update_group(
        aws_retry=True,
        GroupName=group_info["Group"]["GroupName"],
        NewPath=path,
    )
    return True


def detach_policies(connection, module, group_name, policies):
    for policy_arn in policies:
        IAMErrorHandler.deletion_error_handler(f"detach policy {policy_arn} from group")(
            connection.detach_group_policy
        )(aws_retry=True, GroupName=group_name, PolicyArn=policy_arn)


def attach_policies(connection, module, group_name, policies):
    for policy_arn in policies:
        IAMErrorHandler.common_error_handler(f"attach policy {policy_arn} to group")(connection.attach_group_policy)(
            aws_retry=True, GroupName=group_name, PolicyArn=policy_arn
        )


def ensure_managed_policies(connection, module, group_info, managed_policies, purge_policies):
    if managed_policies is None:
        return False

    if managed_policies:
        managed_policies = convert_managed_policy_names_to_arns(connection, managed_policies)

    group_name = group_info["Group"]["GroupName"]

    current_attached_policies_desc = get_attached_policy_list(connection, module, group_name)
    current_attached_policies = [policy["PolicyArn"] for policy in current_attached_policies_desc]

    policies_to_add = list(set(managed_policies) - set(current_attached_policies))
    policies_to_remove = []
    if purge_policies:
        policies_to_remove = list(set(current_attached_policies) - set(managed_policies))

    if not policies_to_add and not policies_to_remove:
        return False

    if module.check_mode:
        return True

    detach_policies(connection, module, group_name, policies_to_remove)
    attach_policies(connection, module, group_name, policies_to_add)

    return True


def add_group_members(connection, module, group_name, members):
    for user in members:
        IAMErrorHandler.common_error_handler(f"add user {user} to group")(connection.add_user_to_group)(
            aws_retry=True, GroupName=group_name, UserName=user
        )


def remove_group_members(connection, module, group_name, members):
    for user in members:
        IAMErrorHandler.deletion_error_handler(f"remove user {user} from group")(connection.remove_user_from_group)(
            aws_retry=True, GroupName=group_name, UserName=user
        )


def ensure_group_members(connection, module, group_info, users, purge_users):
    if users is None:
        return False

    group_name = group_info["Group"]["GroupName"]
    current_group_members = [member["UserName"] for member in group_info["Users"]]

    members_to_add = list(set(users) - set(current_group_members))
    members_to_remove = []
    if purge_users:
        members_to_remove = list(set(current_group_members) - set(users))

    if not members_to_add and not members_to_remove:
        return False

    if module.check_mode:
        return True

    add_group_members(connection, module, group_name, members_to_add)
    remove_group_members(connection, module, group_name, members_to_remove)

    return True


@IAMErrorHandler.common_error_handler("create group")
def get_or_create_group(connection, module, group_name, path):
    group = get_iam_group(connection, group_name)
    if group:
        return False, group

    params = {"GroupName": group_name}
    if path is not None:
        params["Path"] = path

    # Check mode means we would create the group
    if module.check_mode:
        module.exit_json(changed=True, create_params=params)

    group = connection.create_group(aws_retry=True, **params)

    if "Users" not in group:
        group["Users"] = []

    return True, group


def create_or_update_group(connection, module):
    changed, group_info = get_or_create_group(connection, module, module.params["name"], module.params["path"])

    # Update the path if necessary
    changed |= ensure_path(
        connection,
        module,
        group_info,
        module.params["path"],
    )

    # Manage managed policies
    changed |= ensure_managed_policies(
        connection,
        module,
        group_info,
        module.params["managed_policies"],
        module.params["purge_policies"],
    )

    # Manage group memberships
    changed |= ensure_group_members(
        connection,
        module,
        group_info,
        module.params["users"],
        module.params["purge_users"],
    )

    if module.check_mode:
        module.exit_json(changed=changed)

    # Get the group again
    group_info = get_iam_group(connection, module.params["name"])
    policies = get_attached_policy_list(connection, module, module.params["name"])
    group_info["AttachedPolicies"] = policies

    module.exit_json(changed=changed, iam_group=normalize_iam_group(group_info))


@IAMErrorHandler.deletion_error_handler("delete group")
def destroy_group(connection, module):
    group_name = module.params.get("name")

    group = get_iam_group(connection, group_name)

    if not group:
        module.exit_json(changed=False)

    # Check mode means we would remove this group
    if module.check_mode:
        module.exit_json(changed=True)

    # Remove any attached policies otherwise deletion fails
    current_policies_desc = get_attached_policy_list(connection, module, group_name)
    current_policies = [policy["PolicyArn"] for policy in current_policies_desc]
    detach_policies(connection, module, group_name, current_policies)

    # Remove any users in the group otherwise deletion fails
    current_group_members = [user["UserName"] for user in group["Users"]]
    remove_group_members(connection, module, group_name, current_group_members)

    connection.delete_group(aws_retry=True, GroupName=group_name)

    module.exit_json(changed=True)


@IAMErrorHandler.list_error_handler("list policies attached to group")
@AWSRetry.jittered_backoff()
def get_attached_policy_list(connection, module, name):
    paginator = connection.get_paginator("list_attached_group_policies")
    return paginator.paginate(GroupName=name).build_full_result()["AttachedPolicies"]


def main():
    argument_spec = dict(
        name=dict(aliases=["group_name"], required=True),
        path=dict(aliases=["prefix", "path_prefix"]),
        managed_policies=dict(default=[], type="list", aliases=["managed_policy"], elements="str"),
        users=dict(default=[], type="list", elements="str"),
        state=dict(choices=["present", "absent"], required=True),
        purge_users=dict(default=False, type="bool"),
        purge_policies=dict(default=False, type="bool", aliases=["purge_policy", "purge_managed_policies"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    identifier_problem = validate_iam_identifiers(
        "group", name=module.params.get("name"), path=module.params.get("path")
    )
    if identifier_problem:
        module.fail_json(msg=identifier_problem)

    connection = module.client("iam", retry_decorator=AWSRetry.jittered_backoff())

    state = module.params.get("state")

    try:
        if state == "present":
            create_or_update_group(connection, module)
        else:
            destroy_group(connection, module)
    except AnsibleIAMError as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_ethernet_network_control_policy
short_description: Ethernet Network Control Policy configuration for Cisco Intersight
description:
  - Manages Ethernet Network Control Policy configuration on Cisco Intersight.
  - A policy to configure network control settings for ethernet connections on Cisco Intersight managed servers.
  - This policy is applicable only for UCS Servers (FI-Attached).
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/fabric/EthNetworkControlPolicy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Ethernet Network Control Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Ethernet Network Control Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  cdp_enabled:
    description:
      - Enables the CDP on an interface.
      - Cisco Discovery Protocol (CDP) is a proprietary Data Link Layer protocol developed by Cisco Systems.
    type: bool
    default: false
  mac_registration_mode:
    description:
      - Determines the MAC addresses that have to be registered with the switch.
      - nativeVlanOnly - Register only the MAC addresses learned in the native VLAN.
      - allVlans - Register the MAC addresses learned in all VLANs.
    type: str
    choices: ['nativeVlanOnly', 'allVlans']
    default: 'nativeVlanOnly'
  uplink_fail_action:
    description:
      - Determines the state of the virtual interface (vethernet / vfc) on the switch when a suitable uplink is not pinned.
      - linkDown - The vethernet will go down.
      - warning - The vethernet will remain up and will not fail over if uplink connectivity is lost.
      - "Important! If the Action on Uplink is set to Warning, the switch will not fail over if uplink connectivity is lost."
    type: str
    choices: ['linkDown', 'warning']
    default: 'linkDown'
  forge_mac:
    description:
      - Determines if the MAC forging is allowed or denied on an interface.
      - allow - Allows MAC forging on the interface.
      - deny - Denies MAC forging on the interface.
    type: str
    choices: ['allow', 'deny']
    default: 'allow'
  lldp_transmit_enabled:
    description:
      - Determines if the LLDP frames can be transmitted by an interface on the switch.
      - Link Layer Discovery Protocol (LLDP) is a vendor-neutral link layer protocol.
    type: bool
    default: false
  lldp_receive_enabled:
    description:
      - Determines if the LLDP frames can be received by an interface on the switch.
      - Link Layer Discovery Protocol (LLDP) is a vendor-neutral link layer protocol.
    type: bool
    default: false
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create an Ethernet Network Control Policy with default settings
  cisco.intersight.intersight_ethernet_network_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "default-network-control-policy"
    description: "Default Ethernet Network Control policy"
    state: present

- name: Create an Ethernet Network Control Policy with CDP and LLDP enabled
  cisco.intersight.intersight_ethernet_network_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "discovery-enabled-policy"
    description: "Network Control policy with discovery protocols enabled"
    tags:
      - Key: "Environment"
        Value: "Production"
    cdp_enabled: true
    mac_registration_mode: "allVlans"
    uplink_fail_action: "warning"
    forge_mac: "deny"
    lldp_transmit_enabled: true
    lldp_receive_enabled: true
    state: present

- name: Create an Ethernet Network Control Policy with strict security settings
  cisco.intersight.intersight_ethernet_network_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "secure-network-control-policy"
    description: "Secure Network Control policy with MAC forging denied"
    cdp_enabled: false
    mac_registration_mode: "nativeVlanOnly"
    uplink_fail_action: "linkDown"
    forge_mac: "deny"
    lldp_transmit_enabled: false
    lldp_receive_enabled: false
    state: present

- name: Delete an Ethernet Network Control Policy
  cisco.intersight.intersight_ethernet_network_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "secure-network-control-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "discovery-enabled-policy",
        "ObjectType": "fabric.EthNetworkControlPolicy",
        "CdpEnabled": true,
        "MacRegistrationMode": "allVlans",
        "UplinkFailAction": "warning",
        "ForgeMac": "deny",
        "LldpSettings": {
            "TransmitEnabled": true,
            "ReceiveEnabled": true
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        cdp_enabled=dict(type='bool', default=False),
        mac_registration_mode=dict(
            type='str',
            choices=['nativeVlanOnly', 'allVlans'],
            default='nativeVlanOnly'
        ),
        uplink_fail_action=dict(
            type='str',
            choices=['linkDown', 'warning'],
            default='linkDown'
        ),
        forge_mac=dict(
            type='str',
            choices=['allow', 'deny'],
            default='allow'
        ),
        lldp_transmit_enabled=dict(type='bool', default=False),
        lldp_receive_enabled=dict(type='bool', default=False),
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/fabric/EthNetworkControlPolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()

        intersight.api_body['CdpEnabled'] = module.params['cdp_enabled']
        intersight.api_body['MacRegistrationMode'] = module.params['mac_registration_mode']
        intersight.api_body['UplinkFailAction'] = module.params['uplink_fail_action']
        intersight.api_body['ForgeMac'] = module.params['forge_mac']
        intersight.api_body['LldpSettings'] = {
            'TransmitEnabled': module.params['lldp_transmit_enabled'],
            'ReceiveEnabled': module.params['lldp_receive_enabled']
        }

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2018, Ansible Project
# Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

# NOTE:
# This has been vendored from ansible.module_utils.common.file. This code has been removed from there for ansible-core 2.16.

from __future__ import annotations

import fcntl
import os
import stat
import time
import typing as t
from contextlib import contextmanager

if t.TYPE_CHECKING:
    from io import TextIOWrapper


class LockTimeout(Exception):
    pass


class FileLock:
    """
    Currently FileLock is implemented via fcntl.flock on a lock file, however this
    behaviour may change in the future. Avoid mixing lock types fcntl.flock,
    fcntl.lockf and module_utils.common.file.FileLock as it will certainly cause
    unwanted and/or unexpected behaviour
    """

    def __init__(self) -> None:
        self.lockfd: TextIOWrapper | None = None

    @contextmanager
    def lock_file(
        self, path: os.PathLike, tmpdir: os.PathLike, lock_timeout: int | float | None = None
    ) -> t.Generator[None]:
        """
        Context for lock acquisition
        """
        try:
            self.set_lock(path, tmpdir, lock_timeout)
            yield
        finally:
            self.unlock()

    def set_lock(
        self, path: os.PathLike, tmpdir: os.PathLike, lock_timeout: int | float | None = None
    ) -> t.Literal[True]:
        """
        Create a lock file based on path with flock to prevent other processes
        using given path.
        Please note that currently file locking only works when it is executed by
        the same user, for example single user scenarios

        :kw path: Path (file) to lock
        :kw tmpdir: Path where to place the temporary .lock file
        :kw lock_timeout:
            Wait n seconds for lock acquisition, fail if timeout is reached.
            0 = Do not wait, fail if lock cannot be acquired immediately,
            Default is None, wait indefinitely until lock is released.
        :returns: True
        """
        lock_path = os.path.join(tmpdir, f"ansible-{os.path.basename(path)}.lock")
        l_wait = 0.1

        self.lockfd = open(lock_path, "w")

        if lock_timeout is not None and lock_timeout <= 0:
            fcntl.flock(self.lockfd, fcntl.LOCK_EX | fcntl.LOCK_NB)
            os.chmod(lock_path, stat.S_IWRITE | stat.S_IREAD)
            return True

        if lock_timeout:
            e_secs: float = 0
            while e_secs < lock_timeout:
                try:
                    fcntl.flock(self.lockfd, fcntl.LOCK_EX | fcntl.LOCK_NB)
                    os.chmod(lock_path, stat.S_IWRITE | stat.S_IREAD)
                    return True
                except BlockingIOError:
                    time.sleep(l_wait)
                    e_secs += l_wait
                    continue

            self.lockfd.close()
            raise LockTimeout(f"{lock_timeout} sec")

        fcntl.flock(self.lockfd, fcntl.LOCK_EX)
        os.chmod(lock_path, stat.S_IWRITE | stat.S_IREAD)

        return True

    def unlock(self) -> t.Literal[True]:
        """
        Make sure lock file is available for everyone and Unlock the file descriptor
        locked by set_lock

        :returns: True
        """
        if not self.lockfd:
            return True

        try:
            fcntl.flock(self.lockfd, fcntl.LOCK_UN)
            self.lockfd.close()
        except ValueError:  # file wasn't opened, let context manager fail gracefully
            pass

        return True



#!/usr/bin/python

# Copyright (c) 2014, Gabe Mulley <gabe.mulley@gmail.com>
# Copyright (c) 2015, David Wittman <dwittman@gmail.com>
# Copyright (c) 2022, Marius Rieder <marius.rieder@scs.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: alternatives
short_description: Manages alternative programs for common commands
description:
  - Manages symbolic links using the C(update-alternatives) tool.
  - Useful when multiple programs are installed but provide similar functionality (for example, different editors).
author:
  - Marius Rieder (@jiuka)
  - David Wittman (@DavidWittman)
  - Gabe Mulley (@mulby)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
options:
  name:
    description:
      - The generic name of the link.
    type: str
    required: true
  path:
    description:
      - The path to the real executable that the link should point to.
    type: path
  family:
    description:
      - The family groups similar alternatives. This option is available only on RHEL-based distributions.
    type: str
    version_added: 10.1.0
  link:
    description:
      - The path to the symbolic link that should point to the real executable.
      - This option is always required on RHEL-based distributions. On Debian-based distributions this option is required
        when the alternative O(name) is unknown to the system.
    type: path
  priority:
    description:
      - The priority of the alternative. If no priority is given for creation V(50) is used as a fallback.
    type: int
  state:
    description:
      - V(present) - install the alternative (if not already installed), but do not set it as the currently selected alternative
        for the group.
      - V(selected) - install the alternative (if not already installed), and set it as the currently selected alternative
        for the group.
      - V(auto) - install the alternative (if not already installed), and set the group to auto mode. Added in community.general
        5.1.0.
      - V(absent) - removes the alternative. Added in community.general 5.1.0.
    choices: [present, selected, auto, absent]
    default: selected
    type: str
    version_added: 4.8.0
  subcommands:
    description:
      - A list of subcommands.
      - Each subcommand needs a name, a link and a path parameter.
      - Subcommands are also named C(slaves) or C(followers), depending on the version of C(alternatives).
    type: list
    elements: dict
    aliases: ['slaves']
    suboptions:
      name:
        description:
          - The generic name of the subcommand.
        type: str
        required: true
      path:
        description:
          - The path to the real executable that the subcommand should point to.
        type: path
        required: true
      link:
        description:
          - The path to the symbolic link that should point to the real subcommand executable.
        type: path
        required: true
    version_added: 5.1.0
requirements: [update-alternatives]
"""

EXAMPLES = r"""
- name: Correct java version selected
  community.general.alternatives:
    name: java
    path: /usr/lib/jvm/java-7-openjdk-amd64/jre/bin/java

- name: Select java-11-openjdk.x86_64 family
  community.general.alternatives:
    name: java
    family: java-11-openjdk.x86_64
  when: ansible_os_family == 'RedHat'

- name: Alternatives link created
  community.general.alternatives:
    name: hadoop-conf
    link: /etc/hadoop/conf
    path: /etc/hadoop/conf.ansible

- name: Make java 32 bit an alternative with low priority
  community.general.alternatives:
    name: java
    path: /usr/lib/jvm/java-7-openjdk-i386/jre/bin/java
    priority: -10

- name: Install Python 3.5 but do not select it
  community.general.alternatives:
    name: python
    path: /usr/bin/python3.5
    link: /usr/bin/python
    state: present

- name: Install Python 3.5 and reset selection to auto
  community.general.alternatives:
    name: python
    path: /usr/bin/python3.5
    link: /usr/bin/python
    state: auto

- name: keytool is a subcommand of java
  community.general.alternatives:
    name: java
    link: /usr/bin/java
    path: /usr/lib/jvm/java-7-openjdk-amd64/jre/bin/java
    subcommands:
      - name: keytool
        link: /usr/bin/keytool
        path: /usr/lib/jvm/java-7-openjdk-amd64/jre/bin/keytool
"""

import os
import re

from ansible.module_utils.basic import AnsibleModule


class AlternativeState:
    PRESENT = "present"
    SELECTED = "selected"
    ABSENT = "absent"
    AUTO = "auto"

    @classmethod
    def to_list(cls):
        return [cls.PRESENT, cls.SELECTED, cls.ABSENT, cls.AUTO]


class AlternativesModule:
    _UPDATE_ALTERNATIVES = None

    def __init__(self, module):
        self.module = module
        self.result = dict(changed=False, diff=dict(before=dict(), after=dict()))
        self.module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}
        self.messages = []
        self.run()

    @property
    def mode_present(self):
        return self.module.params.get("state") in [
            AlternativeState.PRESENT,
            AlternativeState.SELECTED,
            AlternativeState.AUTO,
        ]

    @property
    def mode_selected(self):
        return self.module.params.get("state") == AlternativeState.SELECTED

    @property
    def mode_auto(self):
        return self.module.params.get("state") == AlternativeState.AUTO

    def run(self):
        self.parse()

        if self.mode_present:
            # Check if we need to (re)install
            subcommands_parameter = self.module.params["subcommands"]
            priority_parameter = self.module.params["priority"]
            if self.path is not None and (
                self.path not in self.current_alternatives
                or (
                    priority_parameter is not None
                    and self.current_alternatives[self.path].get("priority") != priority_parameter
                )
                or (
                    subcommands_parameter is not None
                    and (
                        not all(
                            s in subcommands_parameter for s in self.current_alternatives[self.path].get("subcommands")
                        )
                        or not all(
                            s in self.current_alternatives[self.path].get("subcommands") for s in subcommands_parameter
                        )
                    )
                )
            ):
                self.install()

            # Check if we need to set the preference
            is_same_path = self.path is not None and self.current_path == self.path
            is_same_family = False
            if self.current_path is not None and self.current_path in self.current_alternatives:
                current_alternative = self.current_alternatives[self.current_path]
                is_same_family = current_alternative.get("family") == self.family

            if self.mode_selected and not (is_same_path or is_same_family):
                self.set()

            # Check if we need to reset to auto
            if self.mode_auto and self.current_mode == "manual":
                self.auto()
        else:
            # Check if we need to uninstall
            if self.path in self.current_alternatives:
                self.remove()

        self.result["msg"] = " ".join(self.messages)
        self.module.exit_json(**self.result)

    def install(self):
        if not os.path.exists(self.path):
            self.module.fail_json(msg=f"Specified path {self.path} does not exist")
        if not self.link:
            self.module.fail_json(
                msg="Needed to install the alternative, but unable to do so as we are missing the link"
            )

        cmd = [self.UPDATE_ALTERNATIVES, "--install", self.link, self.name, self.path, str(self.priority)]
        if self.family is not None:
            cmd.extend(["--family", self.family])

        if self.module.params["subcommands"] is not None:
            subcommands = [["--slave", subcmd["link"], subcmd["name"], subcmd["path"]] for subcmd in self.subcommands]
            cmd += [item for sublist in subcommands for item in sublist]

        self.result["changed"] = True
        self.messages.append(f"Install alternative '{self.path}' for '{self.name}'.")

        if not self.module.check_mode:
            self.module.run_command(cmd, check_rc=True)

        if self.module._diff:
            self.result["diff"]["after"] = dict(
                state=AlternativeState.PRESENT,
                path=self.path,
                family=self.family,
                priority=self.priority,
                link=self.link,
            )
            if self.subcommands:
                self.result["diff"]["after"].update(dict(subcommands=self.subcommands))

    def remove(self):
        cmd = [self.UPDATE_ALTERNATIVES, "--remove", self.name, self.path]
        self.result["changed"] = True
        self.messages.append(f"Remove alternative '{self.path}' from '{self.name}'.")

        if not self.module.check_mode:
            self.module.run_command(cmd, check_rc=True)

        if self.module._diff:
            self.result["diff"]["after"] = dict(state=AlternativeState.ABSENT)

    def set(self):
        # Path takes precedence over family as it is more specific
        if self.path is None:
            arg = self.family
        else:
            arg = self.path

        cmd = [self.UPDATE_ALTERNATIVES, "--set", self.name, arg]
        self.result["changed"] = True
        self.messages.append(f"Set alternative '{arg}' for '{self.name}'.")

        if not self.module.check_mode:
            self.module.run_command(cmd, check_rc=True)

        if self.module._diff:
            self.result["diff"]["after"]["state"] = AlternativeState.SELECTED

    def auto(self):
        cmd = [self.UPDATE_ALTERNATIVES, "--auto", self.name]
        self.messages.append(f"Set alternative to auto for '{self.name}'.")
        self.result["changed"] = True

        if not self.module.check_mode:
            self.module.run_command(cmd, check_rc=True)

        if self.module._diff:
            self.result["diff"]["after"]["state"] = AlternativeState.PRESENT

    @property
    def name(self):
        return self.module.params.get("name")

    @property
    def path(self):
        return self.module.params.get("path")

    @property
    def family(self):
        return self.module.params.get("family")

    @property
    def link(self):
        return self.module.params.get("link") or self.current_link

    @property
    def priority(self):
        if self.module.params.get("priority") is not None:
            return self.module.params.get("priority")
        return self.current_alternatives.get(self.path, {}).get("priority", 50)

    @property
    def subcommands(self):
        if self.module.params.get("subcommands") is not None:
            return self.module.params.get("subcommands")
        elif self.path in self.current_alternatives and self.current_alternatives[self.path].get("subcommands"):
            return self.current_alternatives[self.path].get("subcommands")
        return None

    @property
    def UPDATE_ALTERNATIVES(self):
        if self._UPDATE_ALTERNATIVES is None:
            self._UPDATE_ALTERNATIVES = self.module.get_bin_path("update-alternatives", True)
        return self._UPDATE_ALTERNATIVES

    def parse(self):
        self.current_mode = None
        self.current_path = None
        self.current_link = None
        self.current_alternatives = {}

        # Run `update-alternatives --display <name>` to find existing alternatives
        (rc, display_output, dummy) = self.module.run_command([self.UPDATE_ALTERNATIVES, "--display", self.name])

        if rc != 0:
            self.module.debug(f"No current alternative found. '{self.UPDATE_ALTERNATIVES}' exited with {rc}")
            return

        current_mode_regex = re.compile(r"\s-\s(?:status\sis\s)?(\w*)(?:\smode|.)$", re.MULTILINE)
        current_path_regex = re.compile(r"^\s*link currently points to (.*)$", re.MULTILINE)
        current_link_regex = re.compile(r"^\s*link \w+ is (.*)$", re.MULTILINE)
        subcmd_path_link_regex = re.compile(r"^\s*(?:slave|follower) (\S+) is (.*)$", re.MULTILINE)

        alternative_regex = re.compile(
            r"^(\/.*)\s-\s(?:family\s(\S+)\s)?priority\s(\d+)((?:\s+(?:slave|follower).*)*)", re.MULTILINE
        )
        subcmd_regex = re.compile(r"^\s+(?:slave|follower) (.*): (.*)$", re.MULTILINE)

        match = current_mode_regex.search(display_output)
        if not match:
            self.module.debug("No current mode found in output")
            return
        self.current_mode = match.group(1)

        match = current_path_regex.search(display_output)
        if not match:
            self.module.debug("No current path found in output")
        else:
            self.current_path = match.group(1)

        match = current_link_regex.search(display_output)
        if not match:
            self.module.debug("No current link found in output")
        else:
            self.current_link = match.group(1)

        subcmd_path_map = dict(subcmd_path_link_regex.findall(display_output))
        if not subcmd_path_map and self.subcommands:
            subcmd_path_map = {s["name"]: s["link"] for s in self.subcommands}

        for path, family, prio, subcmd in alternative_regex.findall(display_output):
            self.current_alternatives[path] = dict(
                priority=int(prio),
                family=family,
                subcommands=[
                    dict(name=name, path=spath, link=subcmd_path_map.get(name))
                    for name, spath in subcmd_regex.findall(subcmd)
                    if spath != "(null)"
                ],
            )

        if self.module._diff:
            if self.path in self.current_alternatives:
                self.result["diff"]["before"].update(
                    dict(
                        state=AlternativeState.PRESENT,
                        path=self.path,
                        priority=self.current_alternatives[self.path].get("priority"),
                        link=self.current_link,
                    )
                )
                if self.current_alternatives[self.path].get("subcommands"):
                    self.result["diff"]["before"].update(
                        dict(subcommands=self.current_alternatives[self.path].get("subcommands"))
                    )
                if self.current_mode == "manual" and self.current_path != self.path:
                    self.result["diff"]["before"].update(dict(state=AlternativeState.SELECTED))
            else:
                self.result["diff"]["before"].update(dict(state=AlternativeState.ABSENT))


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            path=dict(type="path"),
            family=dict(type="str"),
            link=dict(type="path"),
            priority=dict(type="int"),
            state=dict(
                type="str",
                choices=AlternativeState.to_list(),
                default=AlternativeState.SELECTED,
            ),
            subcommands=dict(
                type="list",
                elements="dict",
                aliases=["slaves"],
                options=dict(
                    name=dict(type="str", required=True),
                    path=dict(type="path", required=True),
                    link=dict(type="path", required=True),
                ),
            ),
        ),
        supports_check_mode=True,
        required_one_of=[("path", "family")],
    )

    AlternativesModule(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: eos_facts
author:
- Peter Sprygada (@privateip)
- Nathaniel Case (@Qalthos)
short_description: Collect facts from remote devices running Arista EOS
description:
- Collects facts from Arista devices running the EOS operating system. This module
  places the facts gathered in the fact tree keyed by the respective resource name.  The
  facts module will always collect a base set of facts from the device and can enable
  or disable collection of additional facts.
version_added: 1.0.0
options:
  gather_subset:
    description:
    - When supplied, this argument will restrict the facts collected to a given subset.  Possible
      values for this argument include C(all), C(hardware), C(config), C(legacy), C(interfaces), and C(min).
      Can specify a list of values to include a larger subset.  Values can also be used
      with an initial C(!) to specify that a specific subset should not be collected.
    required: false
    type: list
    elements: str
    default: 'min'
  gather_network_resources:
    description:
    - When supplied, this argument will restrict the facts collected to a given subset.
      Possible values for this argument include all and the resources like interfaces,
      vlans etc. Can specify a list of values to include a larger subset. Values can
      also be used with an initial C(!) to specify that a specific subset should
      not be collected. Values can also be used with an initial C(!) to specify
      that a specific subset should not be collected. Valid subsets are 'all', 'interfaces',
      'l2_interfaces', 'l3_interfaces', 'lacp', 'lacp_interfaces', 'lag_interfaces',
      'lldp_global', 'lldp_interfaces', 'vlans', 'acls'.
    required: false
    type: list
    elements: str
  available_network_resources:
    description: When 'True' a list of network resources for which resource modules are available will be provided.
    type: bool
    default: false
"""

EXAMPLES = """
- name: Gather all legacy facts
- arista.eos.eos_facts:
    gather_subset: all

- name: Gather only the config and default facts
  arista.eos.eos_facts:
    gather_subset:
      - config

- name: Do not gather hardware facts
  arista.eos.eos_facts:
    gather_subset:
      - '!hardware'

- name: Gather legacy and resource facts
  arista.eos.eos_facts:
    gather_subset: all
    gather_network_resources: all

- name: Gather only the interfaces resource facts and no legacy facts
- arista.eos.eos_facts:
    gather_subset:
      - '!all'
      - '!min'
    gather_network_resources:
      - interfaces

- name: Gather all resource facts and minimal legacy facts
  arista.eos.eos_facts:
    gather_subset: min
    gather_network_resources: all
"""

RETURN = """
ansible_net_gather_subset:
  description: The list of fact subsets collected from the device
  returned: always
  type: list

ansible_net_gather_network_resources:
  description: The list of fact for network resource subsets collected from the device
  returned: when the resource is configured
  type: list

# default
ansible_net_model:
  description: The model name returned from the device
  returned: always
  type: str
ansible_net_serialnum:
  description: The serial number of the remote device
  returned: always
  type: str
ansible_net_version:
  description: The operating system version running on the remote device
  returned: always
  type: str
ansible_net_hostname:
  description: The configured hostname of the device
  returned: always
  type: str
ansible_net_image:
  description: The image file the device is running
  returned: always
  type: str
ansible_net_fqdn:
  description: The fully qualified domain name of the device
  returned: always
  type: str
ansible_net_api:
  description: The name of the transport
  returned: always
  type: str
ansible_net_python_version:
  description: The Python version Ansible controller is using
  returned: always
  type: str

# hardware
ansible_net_filesystems:
  description: All file system names available on the device
  returned: when hardware is configured
  type: list
ansible_net_memfree_mb:
  description: The available free memory on the remote device in Mb
  returned: when hardware is configured
  type: int
ansible_net_memtotal_mb:
  description: The total memory on the remote device in Mb
  returned: when hardware is configured
  type: int

# config
ansible_net_config:
  description: The current active config from the device
  returned: when config is configured
  type: str

# interfaces
ansible_net_all_ipv4_addresses:
  description: All IPv4 addresses configured on the device
  returned: when interfaces is configured
  type: list
ansible_net_all_ipv6_addresses:
  description: All IPv6 addresses configured on the device
  returned: when interfaces is configured
  type: list
ansible_net_interfaces:
  description: A hash of all interfaces running on the system
  returned: when interfaces is configured
  type: dict
ansible_net_neighbors:
  description: The list of LLDP neighbors from the remote device
  returned: when interfaces is configured
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.facts.facts import (
    FactsArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.facts import (
    FACT_RESOURCE_SUBSETS,
    Facts,
)


def main():
    """
    Main entry point for module execution

    :returns: ansible_facts
    """
    argument_spec = FactsArgs.argument_spec

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    warnings = []
    ansible_facts = {}
    if module.params.get("available_network_resources"):
        ansible_facts["available_network_resources"] = sorted(
            FACT_RESOURCE_SUBSETS.keys(),
        )
    result = Facts(module).get_facts()
    additional_facts, additional_warnings = result
    ansible_facts.update(additional_facts)
    warnings.extend(additional_warnings)

    module.exit_json(ansible_facts=ansible_facts, warnings=warnings)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networkfirewall
short_description: manage AWS Network Firewall firewalls
version_added: 4.0.0
description:
  - A module for creating, updating and deleting AWS Network Firewall firewalls.
options:
  arn:
    description:
      - The ARN of the firewall.
      - Exactly one of I(arn) or I(name) must be provided.
    required: false
    type: str
    aliases: ['firewall_arn']
  name:
    description:
      - The name of the firewall.
      - Cannot be updated after creation.
      - Exactly one of I(arn) or I(name) must be provided.
    required: false
    type: str
    aliases: ['firewall_name']
  state:
    description:
      - Create or remove the firewall.
    required: false
    choices: ['present', 'absent']
    default: 'present'
    type: str
  description:
    description:
      - A description for the firewall.
    required: false
    type: str
  delete_protection:
    description:
      - When I(delete_protection=True), the firewall is protected from deletion.
      - Defaults to C(false) when not provided on creation.
    type: bool
    required: false
  policy_change_protection:
    description:
      - When I(policy_change_protection=True), the firewall is protected from
        changes to which policy is attached to the firewall.
      - Defaults to C(false) when not provided on creation.
    type: bool
    required: false
    aliases: ['firewall_policy_change_protection']
  subnet_change_protection:
    description:
      - When I(subnet_change_protection=True), the firewall is protected from
        changes to which subnets is attached to the firewall.
      - Defaults to C(false) when not provided on creation.
    type: bool
    required: false
  wait:
    description:
      - On creation, whether to wait for the firewall to reach the C(READY)
        state.
      - On deletion, whether to wait for the firewall to reach the C(DELETED)
        state.
      - On update, whether to wait for the firewall to reach the C(IN_SYNC)
        configuration synchronization state.
    type: bool
    required: false
    default: true
  wait_timeout:
    description:
      - Maximum time, in seconds, to wait for the firewall to reach the
        expected state.
      - Defaults to 600 seconds.
    type: int
    required: false
  subnets:
    description:
      - The ID of the subnets to which the firewall will be associated.
      - Required when creating a new firewall.
    type: list
    elements: str
    required: false
  purge_subnets:
    description:
      - If I(purge_subnets=true), existing subnets will be removed from the
        firewall as necessary to match exactly what is defined by I(subnets).
    type: bool
    required: false
    default: true
  policy:
    description:
      - The ARN of the Network Firewall policy to use for the firewall.
      - Required when creating a new firewall.
    type: str
    required: false
    aliases: ['firewall_policy_arn']

author:
  - Mark Chappell (@tremble)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Create an AWS Network Firewall
- community.aws.networkfirewall:
    name: 'ExampleFirewall'
    state: present
    policy: 'ExamplePolicy'
    subnets:
      - 'subnet-123456789abcdef01'

# Create an AWS Network Firewall with various options, don't wait for creation
# to finish.
- community.aws.networkfirewall:
    name: 'ExampleFirewall'
    state: present
    delete_protection: true
    description: "An example Description"
    policy: 'ExamplePolicy'
    policy_change_protection: true
    subnets:
      - 'subnet-123456789abcdef01'
      - 'subnet-abcdef0123456789a'
    subnet_change_protection: true
    tags:
      ExampleTag: Example Value
      another_tag: another_example
    wait: false


# Delete an AWS Network Firewall
- community.aws.networkfirewall:
    state: absent
    name: 'ExampleFirewall'
"""

RETURN = r"""
firewall:
  description: The full details of the firewall
  returned: success
  type: dict
  contains:
    firewall:
      description: The details of the firewall
      type: dict
      returned: success
      contains:
        delete_protection:
          description: A flag indicating whether it is possible to delete the firewall.
          type: str
          returned: success
          example: true
        description:
          description: A description of the firewall.
          type: str
          returned: success
          example: "Description"
        firewall_arn:
          description: The ARN of the firewall.
          type: str
          returned: success
          example: "arn:aws:network-firewall:us-east-1:123456789012:firewall/ExampleFirewall"
        firewall_id:
          description: A unique ID for the firewall.
          type: str
          returned: success
          example: "12345678-abcd-1234-abcd-123456789abc"
        firewall_name:
          description: The name of the firewall.
          type: str
          returned: success
          example: "ExampleFirewall"
        firewall_policy_arn:
          description:  The ARN of the firewall policy used by the firewall.
          type: str
          returned: success
          example: "arn:aws:network-firewall:us-east-1:123456789012:firewall-policy/ExamplePolicy"
        firewall_policy_change_protection:
          description:
            - A flag indicating whether it is possible to change which firewall
              policy is used by the firewall.
          type: bool
          returned: success
          example: false
        subnet_change_protection:
          description:
            - A flag indicating whether it is possible to change which subnets
              the firewall endpoints are in.
          type: bool
          returned: success
          example: true
        subnets:
          description: A list of the subnets the firewall endpoints are in.
          type: list
          elements: str
          example: ["subnet-12345678", "subnet-87654321"]
        subnet_mappings:
          description: A list representing the subnets the firewall endpoints are in.
          type: list
          elements: dict
          contains:
            subnet_id:
              description: The ID of the subnet.
              type: str
              returned: success
              example: "subnet-12345678"
        tags:
          description: The tags associated with the firewall.
          type: dict
          returned: success
          example: '{"SomeTag": "SomeValue"}'
        vpc_id:
          description: The ID of the VPC that the firewall is used by.
          type: str
          returned: success
          example: "vpc-0123456789abcdef0"
    firewall_metadata:
      description: Metadata about the firewall
      type: dict
      returned: success
      contains:
        configuration_sync_state_summary:
          description:
            - A short summary of the synchronization status of the
              policy and rule groups.
          type: str
          returned: success
          example: "IN_SYNC"
        status:
          description:
            - A short summary of the status of the firewall endpoints.
          type: str
          returned: success
          example: "READY"
        sync_states:
          description:
            - A description, broken down by availability zone, of the status
              of the firewall endpoints as well as the synchronization status
              of the policies and rule groups.
          type: dict
          returned: success
          example:
            {
              "us-east-1a": {
                "attachment": {
                  "endpoint_id": "vpce-123456789abcdef01",
                  "status": "READY",
                  "subnet_id": "subnet-12345678"
                },
                "config": {
                  "arn:aws:network-firewall:us-east-1:123456789012:firewall-policy/Ansible-Example": {
                    "sync_status": "IN_SYNC",
                    "update_token": "abcdef01-0000-0000-0000-123456789abc"
                  },
                  "arn:aws:network-firewall:us-east-1:123456789012:stateful-rulegroup/ExampleDomainList": {
                    "sync_status": "IN_SYNC",
                    "update_token": "12345678-0000-0000-0000-abcdef012345"
                  }
                }
              }
            }
"""

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.networkfirewall import NetworkFirewallManager


def main():
    argument_spec = dict(
        name=dict(type="str", required=False, aliases=["firewall_name"]),
        arn=dict(type="str", required=False, aliases=["firewall_arn"]),
        state=dict(type="str", required=False, default="present", choices=["present", "absent"]),
        description=dict(type="str", required=False),
        tags=dict(type="dict", required=False, aliases=["resource_tags"]),
        purge_tags=dict(type="bool", required=False, default=True),
        wait=dict(type="bool", required=False, default=True),
        wait_timeout=dict(type="int", required=False),
        subnet_change_protection=dict(type="bool", required=False),
        policy_change_protection=dict(type="bool", required=False, aliases=["firewall_policy_change_protection"]),
        delete_protection=dict(type="bool", required=False),
        subnets=dict(type="list", elements="str", required=False),
        purge_subnets=dict(type="bool", required=False, default=True),
        policy=dict(type="str", required=False, aliases=["firewall_policy_arn"]),
    )

    mutually_exclusive = [
        ["arn", "name"],
    ]
    required_one_of = [
        ["arn", "name"],
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
        required_one_of=required_one_of,
    )

    arn = module.params.get("arn")
    name = module.params.get("name")
    state = module.params.get("state")

    manager = NetworkFirewallManager(module, name=name, arn=arn)
    manager.set_wait(module.params.get("wait", None))
    manager.set_wait_timeout(module.params.get("wait_timeout", None))

    if state == "absent":
        manager.set_delete_protection(module.params.get("delete_protection", None))
        manager.delete()
    else:
        if not manager.original_resource:
            if not module.params.get("subnets", None):
                module.fail_json("The subnets parameter must be provided on creation.")
            if not module.params.get("policy", None):
                module.fail_json("The policy parameter must be provided on creation.")
        manager.set_description(module.params.get("description", None))
        manager.set_tags(module.params.get("tags", None), module.params.get("purge_tags", None))
        manager.set_subnet_change_protection(module.params.get("subnet_change_protection", None))
        manager.set_policy_change_protection(module.params.get("policy_change_protection", None))
        manager.set_delete_protection(module.params.get("delete_protection", None))
        manager.set_subnets(module.params.get("subnets", None), module.params.get("purge_subnets", None))
        manager.set_policy(module.params.get("policy", None))
        manager.flush_changes()

    results = dict(
        changed=manager.changed,
        firewall=manager.updated_resource,
    )
    if manager.changed:
        diff = dict(
            before=manager.original_resource,
            after=manager.updated_resource,
        )
        results["diff"] = diff
    module.exit_json(**results)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c), Michael DeHaan <michael.dehaan@gmail.com>, 2012-2013
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from ansible.module_utils.common.text.converters import to_native
from ansible.module_utils.common.text.converters import to_text


def boto3_tag_list_to_ansible_dict(tags_list, tag_name_key_name=None, tag_value_key_name=None):
    """Convert a boto3 list of resource tags to a flat dict of key:value pairs
    Args:
        tags_list (list): List of dicts representing AWS tags.
        tag_name_key_name (str): Value to use as the key for all tag keys (useful because boto3 doesn't always use "Key")
        tag_value_key_name (str): Value to use as the key for all tag values (useful because boto3 doesn't always use "Value")
    Basic Usage:
        >>> tags_list = [{'Key': 'MyTagKey', 'Value': 'MyTagValue'}]
        >>> boto3_tag_list_to_ansible_dict(tags_list)
        [
            {
                'Key': 'MyTagKey',
                'Value': 'MyTagValue'
            }
        ]
    Returns:
        Dict: Dict of key:value pairs representing AWS tags
         {
            'MyTagKey': 'MyTagValue',
        }
    """

    if tag_name_key_name and tag_value_key_name:
        tag_candidates = {tag_name_key_name: tag_value_key_name}
    else:
        tag_candidates = {"key": "value", "Key": "Value"}

    # minio seems to return [{}] as an empty tags_list
    if not tags_list or not any(tag for tag in tags_list):
        return {}
    for k, v in tag_candidates.items():
        if k in tags_list[0] and v in tags_list[0]:
            return dict((tag[k], tag[v]) for tag in tags_list)
    raise ValueError(f"Couldn't find tag key (candidates {str(tag_candidates)}) in tag list {str(tags_list)}")


def ansible_dict_to_boto3_tag_list(tags_dict, tag_name_key_name="Key", tag_value_key_name="Value"):
    """Convert a flat dict of key:value pairs representing AWS resource tags to a boto3 list of dicts

    Note: booleans are converted to their Capitalized text form ("True" and "False"), this is
    different to ansible_dict_to_boto3_filter_list because historically we've used "to_text()" and
    AWS stores tags as strings, whereas for things which are actually booleans in AWS are returned
    as lowercase strings in filters.

    Args:
        tags_dict (dict): Dict representing AWS resource tags.
        tag_name_key_name (str): Value to use as the key for all tag keys (useful because boto3 doesn't always use "Key")
        tag_value_key_name (str): Value to use as the key for all tag values (useful because boto3 doesn't always use "Value")
    Basic Usage:
        >>> tags_dict = {'MyTagKey': 'MyTagValue'}
        >>> ansible_dict_to_boto3_tag_list(tags_dict)
        {
            'MyTagKey': 'MyTagValue'
        }
    Returns:
        List: List of dicts containing tag keys and values
        [
            {
                'Key': 'MyTagKey',
                'Value': 'MyTagValue'
            }
        ]
    """

    if not tags_dict:
        return []

    tags_list = []
    for k, v in tags_dict.items():
        tags_list.append({tag_name_key_name: k, tag_value_key_name: to_native(v)})

    return tags_list


def _tag_name_to_filter_key(tag_name):
    return f"tag:{tag_name}"


def ansible_dict_to_tag_filter_dict(tags_dict):
    """Prepends "tag:" to all of the keys (not the values) in a dict
    This is useful when you're then going to build a filter including the tags.

    Note: booleans are converted to their Capitalized text form ("True" and "False"), this is
    different to ansible_dict_to_boto3_filter_list because historically we've used "to_text()" and
    AWS stores tags as strings, whereas for things which are actually booleans in AWS are returned
    as lowercase strings in filters.

    Args:
        tags_dict (dict): Dict representing AWS resource tags.

    Basic Usage:
        >>> filters = ansible_dict_to_boto3_filter_list(ansible_dict_to_tag_filter_dict(tags))

    Returns:
        Dict: A dictionary suitable for passing to ansible_dict_to_boto3_filter_list which can
        also be combined with other common filter parameters.
    """
    if not tags_dict:
        return {}
    return {_tag_name_to_filter_key(k): to_native(v) for k, v in tags_dict.items()}


def boto3_tag_specifications(tags_dict, types=None):
    """Converts a list of resource types and a flat dictionary of key:value pairs representing AWS
    resource tags to a TagSpecification object.

    https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_TagSpecification.html

    Args:
        tags_dict (dict): Dict representing AWS resource tags.
        types (list) A list of resource types to be tagged.
    Basic Usage:
        >>> tags_dict = {'MyTagKey': 'MyTagValue'}
        >>> boto3_tag_specifications(tags_dict, ['instance'])
        [
            {
                'ResourceType': 'instance',
                'Tags': [
                    {
                        'Key': 'MyTagKey',
                        'Value': 'MyTagValue'
                    }
                ]
            }
        ]
    Returns:
        List: List of dictionaries representing an AWS Tag Specification
    """
    if not tags_dict:
        return None
    specifications = list()
    tag_list = ansible_dict_to_boto3_tag_list(tags_dict)

    if not types:
        specifications.append(dict(Tags=tag_list))
        return specifications

    if isinstance(types, str):
        types = [types]

    for type_name in types:
        specifications.append(dict(ResourceType=type_name, Tags=tag_list))

    return specifications


def compare_aws_tags(current_tags_dict, new_tags_dict, purge_tags=True):
    """
    Compare two dicts of AWS tags. Dicts are expected to of been created using 'boto3_tag_list_to_ansible_dict' helper function.
    Two dicts are returned - the first is tags to be set, the second is any tags to remove. Since the AWS APIs differ
    these may not be able to be used out of the box.

    :param current_tags_dict:
    :param new_tags_dict:
    :param purge_tags:
    :return: tag_key_value_pairs_to_set: a dict of key value pairs that need to be set in AWS. If all tags are identical this dict will be empty
    :return: tag_keys_to_unset: a list of key names (type str) that need to be unset in AWS. If no tags need to be unset this list will be empty
    """

    tag_key_value_pairs_to_set = {}
    tag_keys_to_unset = []

    if purge_tags:
        for key in current_tags_dict.keys():
            if key in new_tags_dict:
                continue
            # Amazon have reserved 'aws:*' tags, we should avoid purging them as
            # this probably isn't what people want to do...
            if key.startswith("aws:"):
                continue
            tag_keys_to_unset.append(key)

    for key in set(new_tags_dict.keys()) - set(tag_keys_to_unset):
        if to_text(new_tags_dict[key]) != current_tags_dict.get(key):
            tag_key_value_pairs_to_set[key] = new_tags_dict[key]

    return tag_key_value_pairs_to_set, tag_keys_to_unset



#!/usr/bin/python
#
# Copyright (c) 2025 Klaas Demter (@Klaas-)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_monitoractiongroups_info
version_added: "3.7.0"
short_description: Get Action Groups
description:
    - Get Action Groups

options:
    name:
        description:
            - The name of the action group you're trying to get details about.
        type: str
    resource_group:
        description:
            - The name of the resource group in which the action group is (if you use name)
            - The name of the resource group where you want to list Action Groups (if you don't use name)
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Klaas Demter (@Klaas-)
'''

EXAMPLES = '''
- name: Get action group details
  azure.azcollection.azure_rm_monitoractiongroups_info:
    name: DCRName
    resource_group: Resource_Group_Name

- name: Get all Action Groups in specific resource group
  azure.azcollection.azure_rm_monitoractiongroups_info:
    resource_group: Resource_Group_Name

- name: Get all Action Groups in the current subscription
  azure.azcollection.azure_rm_monitoractiongroups_info:
'''

RETURN = '''
actiongroups:
    description:
        - List of Action Groups
        - Can be empty if listing Action Groups
    type: list
    returned: always
    sample: [
        {
            "arm_role_receivers": [],
            "automation_runbook_receivers": [],
            "azure_app_push_receivers": [],
            "azure_function_receivers": [],
            "email_receivers": [
                {
                    "email_address": "mail@example.com",
                    "name": "sendmailtoexample",
                    "status": "Enabled",
                    "use_common_alert_schema": true
                }
            ],
            "enabled": true,
            "event_hub_receivers": [],
            "group_short_name": "aaa",
            "id": \
"/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/resource_group_name/providers/microsoft.insights/actionGroups/action_group_name",
            "itsm_receivers": [],
            "location": "Global",
            "logic_app_receivers": [],
            "name": "action_group_name",
            "sms_receivers": [],
            "tags": {},
            "type": "Microsoft.Insights/ActionGroups",
            "voice_receivers": [],
            "webhook_receivers": []
        }
    ]
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError, HttpResponseError

except ImportError:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'actiongroups'


class AzureRMactiongroupsInfo(AzureRMModuleBase):
    """Information class for an Azure RM Action Groups"""

    def __init__(self):
        self.module_arg_spec = dict(
            name=dict(type='str', required=False),
            resource_group=dict(type='str', required=False),
        )

        self.required_by = {
            'name': 'resource_group'
        }

        self.name = None
        self.resource_group = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            changed=False,
            actiongroups=[]
        )

        super(AzureRMactiongroupsInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                      supports_check_mode=True,
                                                      supports_tags=False,
                                                      facts_module=True,
                                                      required_by=self.required_by)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            result = self.get_action_group()
        else:
            result = self.list_action_groups()

        self.results['actiongroups'] = result

        return self.results

    def get_action_group(self):
        '''
        Gets the properties of the specified action group.

        :return: List of Action Groups
        '''
        self.log("Checking if action group {0} in resource group {1} is present".format(self.name,
                                                                                        self.resource_group))

        result = []
        action_group = None

        try:
            action_group = self.monitor_management_client_action_groups.action_groups.get(action_group_name=self.name,
                                                                                          resource_group_name=self.resource_group)
        except ResourceNotFoundError as ex:
            self.log("Could not find action group {0} in resource group {1}".format(self.name, self.resource_group))
            return []
        except HttpResponseError as ex:
            if ex.error.code == 'InvalidSubscriptionId':
                self.log("Could not find subscription id")
                return []
            else:
                raise Exception(ex)
        if action_group:
            result = [self.serialize_obj(action_group, AZURE_OBJECT_CLASS)]

        return result

    def list_action_groups(self):
        '''
        Gets the properties of the specified Action Groups in resource group or subscription.

        :return: List of Action Groups
        '''

        result = []
        action_groups = None

        if self.resource_group:
            self.log("Checking if the Action Groups in resource group {0} are present".format(self.resource_group))
            action_groups_mgmt_client = self.monitor_management_client_action_groups.action_groups
            action_groups = action_groups_mgmt_client.list_by_resource_group(resource_group_name=self.resource_group)
        else:
            self.log("Checking if the Action Groups are present in subscription")
            action_groups = self.monitor_management_client_action_groups.action_groups.list_by_subscription_id()
        if action_groups:
            # it seems the exception is thrown when iterating through action_groups, not when setting action_groups
            try:
                for item in action_groups:
                    result.append(self.serialize_obj(item, AZURE_OBJECT_CLASS))
            except ResourceNotFoundError as ex:
                self.log("Could not find resource group {0}".format(self.resource_group))
            except HttpResponseError as ex:
                if ex.error.code == 'InvalidSubscriptionId':
                    self.log("Could not find subscription id")
                else:
                    raise Exception(ex)

        return result


def main():
    """Main execution"""
    AzureRMactiongroupsInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_l3_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_l3_interfaces
short_description: L3 interfaces resource module
description: This module manages Layer-3 interfaces attributes of NX-OS Interfaces.
version_added: 1.0.0
author:
  - Trishna Guha (@trishnaguha)
  - Nikhil Bhasin (@nickbhasin)
notes:
- Tested against NXOS 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
options:
  config:
    description: A dictionary of Layer-3 interface options
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Full name of L3 interface, i.e. Ethernet1/1.
        type: str
        required: true
      mac_address:
        description: >-
          Manually set interface MAC address or extract the MAC address (3) from the
          IPv6 address configured on the interface.
        type: str
      bandwidth:
        description: Manually set the bandwidth
        type: dict
        suboptions:
          kilobits:
            description: Bandwidth in kilobits
            type: int
          inherit:
            description: Specify that bandwidth is inherited
            type: bool
      dot1q:
        description:
        - Configures IEEE 802.1Q VLAN encapsulation on a subinterface.
        type: int
      evpn_multisite_tracking:
        description:
        -  VxLAN evpn multisite Interface tracking. Supported only on selected model.
        type: str
        version_added: 1.1.0
        choices:
        - fabric-tracking
        - dci-tracking
      ipv6_redirects:
            description:
            - Enables/disables ipv6 redirects.
            type: bool
      ipv6_unreachables:
            description:
            - Enables/disables ip redirects.
            type: bool
      redirects:
            description:
            - Enables/disables ipv4 redirects.
            type: bool
      unreachables:
            description:
            - Enables/disables ip redirects.
            type: bool
      proxy_arp:
            description: Configure proxy ARP.
            type: bool
      port_unreachable:
        description: Enable sending ICMP port-unreachable.
        type: bool
      verify:
        description: Configure Unicast Reverse Path Forwarding or IP Source Guard.
        type: dict
        suboptions:
          unicast:
            description: Unicast Reverse Path Forwarding.
            type: dict
            suboptions:
              source:
                description: Validation of source address.
                type: dict
                suboptions:
                  reachable_via:
                    description: Specify reachability check to apply to the source address.
                    type: dict
                    suboptions:
                      mode:
                        description: Source is reachable via any/rx interface.
                        type: str
                      allow_default:
                        description: Loose Default Route Unicast Reverse Path Forwarding.
                        type: bool
      ipv6_verify:
        description: Unicast Reverse Path Forwarding.
        type: dict
        suboptions:
          unicast:
            description: Unicast Reverse Path Forwarding.
            type: dict
            suboptions:
              source:
                description: Validation of source address.
                type: dict
                suboptions:
                  reachable_via:
                    description: Specify reachability check to apply to the source address.
                    type: dict
                    suboptions:
                      mode:
                        description: Source is reachable via any/rx interface.
                        type: str
                      allow_default:
                        description: Loose Default Route Unicast Reverse Path Forwarding.
                        type: bool
      dhcp:
        description: Configure DHCP snooping or relay
        type: dict
        suboptions:
          ipv4:
            description: DHCP snooping for ipv4
            type: dict
            suboptions:
              option82:
                description: DHCP option82.
                type: dict
                suboptions:
                  suboption:
                    description: DHCP option82.
                    type: dict
                    suboptions:
                      circuit_id:
                        description: DHCP option82 suboption circuit-id string configuration.
                        type: str
              smart_relay:
                description: Configure DHCP smart relay on interface.
                type: bool
              relay:
                description: Configure relay agent.
                type: dict
                suboptions:
                  information:
                    description: Relay agent information option.
                    type: dict
                    suboptions:
                      trusted:
                        description: Enable relay trust on this interface.
                        type: bool
                  subnet_selection:
                    description: Configure gateway address for DHCP relay
                    type: dict
                    suboptions:
                      subnet_ip:
                        description: IP address
                        type: str
                  source_interface:
                    description: Configure gateway address for DHCP relay
                    type: dict
                    suboptions:
                      interface_type:
                        description: Type of interface
                        type: str
                      interface_id:
                        description: Interface ID
                        type: str
                  address:
                    description: List of ipv4 relay addresses
                    type: list
                    elements: dict
                    suboptions:
                      relay_ip:
                        description: IP address
                        type: str
                      vrf_name:
                        description: Helper address VRF membership
                        type: str
          ipv6:
            description: DHCP snooping for ipv6
            type: dict
            suboptions:
              smart_relay:
                description: Configure DHCP smart relay on interface.
                type: bool
              relay:
                description: Configure relay agent.
                type: dict
                suboptions:
                  source_interface:
                    description: Configure source interface for DHCPv6 relay.
                    type: dict
                    suboptions:
                      interface_type:
                        description: Type of interface
                        type: str
                      interface_id:
                        description: Interface ID
                        type: str
                  address:
                    description: Configure DHCPv6 server relay address
                    type: list
                    elements: dict
                    suboptions:
                      relay_ip:
                        description: IP address
                        type: str
                      vrf_name:
                        description: Helper address VRF membership
                        type: str
                      interface_type:
                        description: Type of interface
                        type: str
                      interface_id:
                        description: Interface ID
                        type: str
      ipv4:
        description: IPv4 address and attributes of the L3 interface.
        type: list
        elements: dict
        suboptions:
          address:
            description:
            - IPV4 address of the L3 interface.
            type: str
          tag:
            description:
            - URIB route tag value for local/direct routes.
            type: int
          secondary:
            description:
            - A boolean attribute to manage addition of secondary IP address.
            type: bool
          ip_network_mask:
                description: IP prefix and network mask length in format x.x.x.x/m or IP network mask in format m.m.m.m
                type: str
          route_preference:
                description: URIB route preference for local/direct routes
                type: int
      ipv6:
        description: IPv6 address and attributes of the L3 interface.
        type: list
        elements: dict
        suboptions:
          address:
            description: IPV6 address of the L3 interface.
            type: str
          tag:
            description: URIB route tag value for local/direct routes.
            type: int
          aggregate_prefix_length:
            description: Prefix-Length for AM Route Aggregation
            type: int
          anycast:
            description: Configure IPv6 anycast address on interface
            type: bool
          eui64:
            description: Configure Extended Unique Identifier for the low-order 64 bits
            type: bool
          route_preference:
            description: URIB route preference for local/direct routes
            type: int
          use_bia:
            description: Use BIA
            type: bool
  running_config:
    description:
      - This option is used only with state I(parsed).
      - >-
        The value of this option should be the output received from the IOS
        device by executing the command B(show running-config | section
        ^interface).
      - >-
        The state I(parsed) reads the configuration from C(running_config)
        option and transforms it into Ansible structured data as per the
        resource module's argspec and the value is then returned in the
        I(parsed) key within the result.
    type: str
  state:
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - parsed
    default: merged
    description:
      - The state the configuration should be left in
      - >-
        The states I(rendered), I(gathered) and I(parsed) does not perform any
        change on the device.
      - >-
        The state I(rendered) will transform the configuration in C(config)
        option to platform specific CLI commands which will be returned in the
        I(rendered) key within the result. For state I(rendered) active
        connection to remote host is not required.
      - >-
        The state I(gathered) will fetch the running configuration from device
        and transform it into structured data in the format as per the resource
        module argspec and the value is returned in the I(gathered) key within
        the result.
      - >-
        The state I(parsed) reads the configuration from C(running_config)
        option and transforms it into JSON format as per the resource module
        parameters and the value is returned in the I(parsed) key within the
        result. The value of C(running_config) option should be the same format
        as the output of command I(show running-config | section ^interface)
        executed on device. For state I(parsed) active connection to remote host
        is not required.
    type: str
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# router# show running-config | section interface
# interface Ethernet1/1
# interface Ethernet1/2

- name: Merge provided configuration with device configuration.
  cisco.nxos.nxos_l3_interfaces:
    config:
      - name: Ethernet1/1
        mac_address: 0011.2233.4455
        ipv4:
          verify:
            unicast:
              source:
                reachable_via:
                  mode: any
                  allow_default: true
          dhcp:
            relay:
              address:
                - relay_ip: 11.0.0.1
                  vrf_name: abc
      - name: Ethernet1/2
        ipv6:
          addresses:
            - ipv6_address: 2001:db8::1/32
              route_preference: 70
              tag: 97
          dhcp:
            relay:
              address:
                - relay_ip: 2001:db8::1:abcd
    state: merged

# Task Output
# -----------
#
# before:
# - name: Ethernet1/1
# - name: Ethernet1/2
# commands:
# - interface Ethernet1/1
# - mac-address 0011.2233.4455
# - ip verify unicast source reachable-via any allow-default
# - ip dhcp relay address 11.0.0.1 use-vrf abc
# - interface Ethernet1/2
# - ipv6 address 2001:db8::1/32 route-preference 70 tag 97
# - ipv6 dhcp relay address 2001:db8::1:abcd
# after:
# - name: Ethernet1/1
#   mac_address: 0011.2233.4455
#   ipv4:
#     verify:
#       unicast:
#         source:
#           reachable_via:
#             mode: any
#             allow_default: true
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 11.0.0.1
#             vrf_name: abc
# - name: Ethernet1/2
#   ipv6:
#     addresses:
#       - ipv6_address: 2001:db8::1/32
#         route_preference: 70
#         tag: 97
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 2001:db8::1:abcd

# After state:
# ------------
#
# router# show running-config | section interface
# interface Ethernet1/1
#  mac-address 0011.2233.4455
#  ip verify unicast source reachable-via any allow-default
#  ip dhcp relay address 11.0.0.1 use-vrf abc
# interface Ethernet1/2
#  ipv6 address 2001:db8::1/32 route-preference 70 tag 97
#  ipv6 dhcp relay address 2001:db8::1:abcd


# Using replaced

# Before state:
# -------------
#
# router# show running-config | section interface
# interface Ethernet 1/1
#  mac-address 00:11:22:33:44:55
#  ip verify unicast source reachable-via any allow-default
#  ip dhcp relay address 11.0.0.1 use-vrf abc
# interface Ethernet 1/2
#  ipv6 dhcp relay address 2001:0db8::1:abcd
#  ipv6 address 2001:db8::1/32 route-preference 70 tag 97

- name: Replace device configuration of specified L3 interfaces with provided configuration.
  cisco.nxos.nxos_l3_interfaces:
    config:
      - name: Ethernet1/2
        mac_address: 0011.2233.4456
        ipv6:
          addresses:
            - ipv6_address: 2001:db8::1/32
              route_preference: 200
              tag: 22
          dhcp:
            relay:
              address:
                - relay_ip: 2001:db8::1:abcd
    state: replaced

# Task Output
# -----------
#
# before:
# - name: Ethernet1/1
#   mac_address: 0011.2233.4455
#   ipv4:
#     verify:
#       unicast:
#         source:
#           reachable_via:
#             mode: any
#             allow_default: true
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 11.0.0.1
#             vrf_name: abc
# - name: Ethernet1/2
#   ipv6:
#     addresses:
#       - ipv6_address: 2001:db8::1/32
#         route_preference: 70
#         tag: 97
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 2001:db8::1:abcd
# commands:
# - interface Ethernet1/2
# - no ipv6 address 2001:db8::1/32 route-preference 70 tag 97
# - ipv6 address 2001:db8::1/32 route-preference 200 tag 22
# - mac-address 0011.2233.4456
# after:
# - name: Ethernet1/1
#   mac_address: 0011.2233.4455
#   ipv4:
#     verify:
#       unicast:
#         source:
#           reachable_via:
#             mode: any
#             allow_default: true
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 11.0.0.1
#             vrf_name: abc
# - name: Ethernet1/2
#   mac_address: 0011.2233.4456
#   ipv6:
#     addresses:
#       - ipv6_address: 2001:db8::1/32
#         route_preference: 200
#         tag: 22
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 2001:db8::1:abcd

# After state:
# ------------
#
# router# show running-config | section interface
# interface Ethernet1/1
#  mac-address 0011.2233.4455
#  ip verify unicast source reachable-via any allow-default
#  ip dhcp relay address 11.0.0.1 use-vrf abc
# interface Ethernet1/2
#  mac-address 0011.2233.4456
#  ipv6 address 2001:db8::1/32 route-preference 200 tag 22
#  ipv6 dhcp relay address 2001:db8::1:abcd

# Using overridden

# Before state:
# -------------
#
# router# show running-config | section interface
# interface Ethernet 1/1
#  mac-address 00:11:22:33:44:55
#  ip verify unicast source reachable-via any allow-default
#  ip dhcp relay address 11.0.0.1 use-vrf abc
# interface Ethernet 1/2
#  ipv6 dhcp relay address 2001:0db8::1:abcd
#  ipv6 address 2001:db8::1/32 route-preference 70 tag 97

- name: Override device configuration with provided configuration.
  cisco.nxos.nxos_l3_interfaces:
    config:
      - name: Ethernet1/1
        mac_address: 0011.2233.4455
        ipv4:
          verify:
            unicast:
              source:
                reachable_via:
                  mode: any
                  allow_default: true
    state: overridden

# Task Output
# -----------
#
# before:
# before:
# - name: Ethernet1/1
#   mac_address: 0011.2233.4455
#   ipv4:
#     verify:
#       unicast:
#         source:
#           reachable_via:
#             mode: any
#             allow_default: true
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 11.0.0.1
#             vrf_name: abc
# - name: Ethernet1/2
#   ipv6:
#     addresses:
#       - ipv6_address: 2001:db8::1/32
#         route_preference: 70
#         tag: 97
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 2001:db8::1:abcd
# commands:
# - interface Ethernet1/1
# - no ip dhcp relay address 11.0.0.1 use-vrf abc
# - interface Ethernet1/2
# - no ipv6 address 2001:db8::1/32 route-preference 70 tag 97
# - no ipv6 dhcp relay address 2001:db8::1:abcd
# after:
# - name: Ethernet1/1
#   mac_address: 0011.2233.4455
#   ipv4:
#     verify:
#       unicast:
#         source:
#           reachable_via:
#             mode: any
#             allow_default: true
# - name: Ethernet1/2

# After state:
# ------------
#
# router# show running-config | section interface
# interface Ethernet1/1
#  mac-address 0011.2233.4455
#  ip verify unicast source reachable-via any allow-default

# Using deleted

# Before state:
# -------------
#
# router# show running-config | section interface
# interface Ethernet 1/1
#  mac-address 00:11:22:33:44:55
#  ip verify unicast source reachable-via any allow-default
#  ip dhcp relay address 11.0.0.1 use-vrf abc
# interface Ethernet 1/2
#  ipv6 dhcp relay address 2001:0db8::1:abcd
#  ipv6 address 2001:db8::1/32 route-preference 70 tag 97

- name: Delete L3 attributes of given interfaces (This won't delete the interface
    itself).
  cisco.nxos.nxos_l3_interfaces:
    config:
      - name: Ethernet1/1
      - name: Ethernet1/2
    state: deleted

# Task Output
# -----------
#
# before:
# - name: Ethernet1/1
#   mac_address: 0011.2233.4455
#   ipv4:
#     verify:
#       unicast:
#         source:
#           reachable_via:
#             mode: any
#             allow_default: true
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 11.0.0.1
#             vrf_name: abc
# - name: Ethernet1/2
#   ipv6:
#     addresses:
#       - ipv6_address: 2001:db8::1/32
#         route_preference: 70
#         tag: 97
#     dhcp:
#       relay:
#         address:
#           - relay_ip: 2001:db8::1:abcd
# commands:
# - interface Ethernet1/1
# - no mac-address 0011.2233.4455
# - no ip verify unicast source reachable-via any allow-default
# - no ip dhcp relay address 11.0.0.1 use-vrf abc
# - interface Ethernet1/2
# - no ipv6 address 2001:db8::1/32 route-preference 70 tag 97
# - no ipv6 dhcp relay address 2001:db8::1:abcd
# after:
# - name: Ethernet1/1
# - name: Ethernet1/2

# After state:
# ------------
#
# router# show running-config | section interface
# interface Ethernet1/1
# interface Ethernet1/2

# Using rendered

- name: Use rendered state to convert task input to device specific commands
  cisco.nxos.nxos_l3_interfaces:
    config:
      - name: Ethernet1/1
        mac_address: 0011.2233.4455
        ipv4:
          verify:
            unicast:
              source:
                reachable_via:
                  mode: any
                  allow_default: true
          dhcp:
            relay:
              address:
                - relay_ip: 11.0.0.1
                  vrf_name: abc
      - name: Ethernet1/2
        ipv6:
          addresses:
            - ipv6_address: 2001:db8::1/32
              route_preference: 70
              tag: 97
          dhcp:
            relay:
              address:
                - relay_ip: 2001:db8::1:abcd
    state: rendered

# Task Output
# -----------
#
# rendered:
# - interface Ethernet1/1
# - mac-address 0011.2233.4455
# - ip verify unicast source reachable-via any allow-default
# - ip dhcp relay address 11.0.0.1 use-vrf abc
# - interface Ethernet1/2
# - ipv6 address 2001:db8::1/32 route-preference 70 tag 97
# - ipv6 dhcp relay address 2001:db8::1:abcd

# Using parsed

# parsed.cfg
# ----------
#
# interface Ethernet1/1
#  mac-address 0011.2233.4455
#  ip verify unicast source reachable-via any allow-default
#  ip dhcp relay address 11.0.0.1 use-vrf abc
# interface Ethernet1/2
#  ipv6 address 2001:db8::1/32 route-preference 70 tag 97
#  ipv6 dhcp relay address 2001:db8::1:abcd

- name: Use parsed state to convert externally supplied config to structured format
  cisco.nxos.nxos_l3_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task output
# -----------
#
# parsed:
#   - name: Ethernet1/1
#     mac_address: 0011.2233.4455
#     ipv4:
#       verify:
#         unicast:
#           source:
#             reachable_via:
#               mode: any
#               allow_default: true
#       dhcp:
#         relay:
#           address:
#             - relay_ip: 11.0.0.1
#               vrf_name: abc
#   - name: Ethernet1/2
#     ipv6:
#       addresses:
#         - ipv6_address: 2001:db8::1/32
#           route_preference: 70
#           tag: 97
#       dhcp:
#         relay:
#           address:
#             - relay_ip: 2001:db8::1:abcd

# Using gathered

# Before state:
# -------------
#
# interface Ethernet 1/1
#  mac-address 00:11:22:33:44:55
#  ip verify unicast source reachable-via any allow-default
#  ip dhcp relay address 11.0.0.1 use-vrf abc
# interface Ethernet 1/2
#  ipv6 dhcp relay address 2001:0db8::1:abcd
#  ipv6 address 2001:db8::1/32 route-preference 70 tag 97

- name: Gather l3_interfaces facts from the device using nxos_l3_interfaces
  cisco.nxos.nxos_l3_interfaces:
    state: gathered

# Task output
# -----------
#
# gathered:
#   - name: Ethernet1/1
#     mac_address: 0011.2233.4455
#     ipv4:
#       verify:
#         unicast:
#           source:
#             reachable_via:
#               mode: any
#               allow_default: true
#       dhcp:
#         relay:
#           address:
#             - relay_ip: 11.0.0.1
#               vrf_name: abc
#   - name: Ethernet1/2
#     ipv6:
#       addresses:
#         - ipv6_address: 2001:db8::1/32
#           route_preference: 70
#           tag: 97
#       dhcp:
#         relay:
#           address:
#             - relay_ip: 2001:db8::1:abcd
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - ip dhcp relay address 11.0.0.1 use-vrf abc
    - ipv6 address 2001:db8::1/32 route-preference 70 tag 97
    - ipv6 dhcp relay address 2001:db8::1:abcd
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - ip dhcp relay address 11.0.0.1 use-vrf abc
    - ipv6 address 2001:db8::1/32 route-preference 70 tag 97
    - ipv6 dhcp relay address 2001:db8::1:abcd
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.l3_interfaces.l3_interfaces import (
    L3_interfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.l3_interfaces.l3_interfaces import (
    L3_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=L3_interfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = L3_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_set_compliance_settings
short_description: Edit existing Compliance Settings.
description:
  - Edit existing Compliance Settings.
  - All operations are performed over Web Services API.
  - Available from R82.10 management version.
version_added: "6.7.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  automatic_scan_scheduler:
    description:
      - Schedule for an automatic full Compliance scan.
    type: dict
    suboptions:
      scan_day:
        description:
          - The scheduled day of the week for the Compliance scan. The default value is 'every_day'.
        type: str
        choices: ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'every_day']
      scan_time:
        description:
          - The scheduled time of day for the Compliance scan in format, "HH:mm:ss".
          - The default value is "23:59:59".
        type: str
      scheduled_scan_on:
        description:
          - Enables or disables the scheduled scan. The default value is true.
        type: bool
  enable_email_alerts:
    description:
      - Enables or disables sending email alerts to SmartEvent (only for alerts). The default value is true.
    type: bool
  enable_smart_event_logs:
    description:
      - Enables or disables sending logs to SmartEvent. The default value is true.
    type: bool
  initialize_best_practices:
    description:
      - If true, creates all the default Best Practices again.
      - After the first scan completes, the value of this parameter is automatically set to false.
      - The default value is true for initial setup and false after first scan.
    type: bool
  partial_scan_delay:
    description:
      - Controls when the partial scan starts after publishing a session. The partial scan checks only the relevant firewall best practices.
      - If the value is < 0, the partial scan is disabled.
      - If the value is 0, the partial scan starts immediately after publishing.
      - If the value is > 0, the partial scan is delayed by the specified number of seconds after publishing.
      - The default value is '0'.
    type: int
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: set-compliance-settings
  cp_mgmt_set_compliance_settings:
    automatic_scan_scheduler:
      scan_day: sunday
      scan_time: 08:00:00
      scheduled_scan_on: false
    partial_scan_delay: -1
"""

RETURN = """
cp_mgmt_set_compliance_settings:
  description: The checkpoint set-compliance-settings output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        automatic_scan_scheduler=dict(type='dict', options=dict(
            scan_day=dict(type='str', choices=['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'every_day']),
            scan_time=dict(type='str'),
            scheduled_scan_on=dict(type='bool')
        )),
        enable_email_alerts=dict(type='bool'),
        enable_smart_event_logs=dict(type='bool'),
        initialize_best_practices=dict(type='bool'),
        partial_scan_delay=dict(type='int')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "set-compliance-settings"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipa_sudocmdgroup
author: Thomas Krahn (@Nosmoht)
short_description: Manage FreeIPA sudo command group
description:
  - Add, modify or delete sudo command group within IPA server using IPA API.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  cn:
    description:
      - Sudo Command Group.
    aliases: ['name']
    required: true
    type: str
  description:
    description:
      - Group description.
    type: str
  state:
    description: State to ensure.
    default: present
    choices: ['absent', 'disabled', 'enabled', 'present']
    type: str
  sudocmd:
    description:
      - List of sudo commands to assign to the group.
      - If an empty list is passed all assigned commands are removed from the group.
      - If option is omitted sudo commands are not checked nor changed.
    type: list
    elements: str
extends_documentation_fragment:
  - community.general._ipa.documentation
  - community.general._ipa.connection_notes
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Ensure sudo command group exists
  community.general.ipa_sudocmdgroup:
    name: group01
    description: Group of important commands
    sudocmd:
      - su
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure sudo command group does not exist
  community.general.ipa_sudocmdgroup:
    name: group01
    state: absent
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret
"""

RETURN = r"""
sudocmdgroup:
  description: Sudo command group as returned by IPA API.
  returned: always
  type: dict
"""

import traceback

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._ipa import IPAClient, ipa_argument_spec


class SudoCmdGroupIPAClient(IPAClient):
    def __init__(self, module, host, port, protocol):
        super().__init__(module, host, port, protocol)

    def sudocmdgroup_find(self, name):
        return self._post_json(method="sudocmdgroup_find", name=None, item={"all": True, "cn": name})

    def sudocmdgroup_add(self, name, item):
        return self._post_json(method="sudocmdgroup_add", name=name, item=item)

    def sudocmdgroup_mod(self, name, item):
        return self._post_json(method="sudocmdgroup_mod", name=name, item=item)

    def sudocmdgroup_del(self, name):
        return self._post_json(method="sudocmdgroup_del", name=name)

    def sudocmdgroup_add_member(self, name, item):
        return self._post_json(method="sudocmdgroup_add_member", name=name, item=item)

    def sudocmdgroup_add_member_sudocmd(self, name, item):
        return self.sudocmdgroup_add_member(name=name, item={"sudocmd": item})

    def sudocmdgroup_remove_member(self, name, item):
        return self._post_json(method="sudocmdgroup_remove_member", name=name, item=item)

    def sudocmdgroup_remove_member_sudocmd(self, name, item):
        return self.sudocmdgroup_remove_member(name=name, item={"sudocmd": item})


def get_sudocmdgroup_dict(description=None):
    data = {}
    if description is not None:
        data["description"] = description
    return data


def get_sudocmdgroup_diff(client, ipa_sudocmdgroup, module_sudocmdgroup):
    return client.get_diff(ipa_data=ipa_sudocmdgroup, module_data=module_sudocmdgroup)


def ensure(module, client):
    name = module.params["cn"]
    state = module.params["state"]
    sudocmd = module.params["sudocmd"]

    module_sudocmdgroup = get_sudocmdgroup_dict(description=module.params["description"])
    ipa_sudocmdgroup = client.sudocmdgroup_find(name=name)

    changed = False
    if state == "present":
        if not ipa_sudocmdgroup:
            changed = True
            if not module.check_mode:
                ipa_sudocmdgroup = client.sudocmdgroup_add(name=name, item=module_sudocmdgroup)
        else:
            diff = get_sudocmdgroup_diff(client, ipa_sudocmdgroup, module_sudocmdgroup)
            if len(diff) > 0:
                changed = True
                if not module.check_mode:
                    data = {}
                    for key in diff:
                        data[key] = module_sudocmdgroup.get(key)
                    client.sudocmdgroup_mod(name=name, item=data)

        if sudocmd is not None:
            changed = client.modify_if_diff(
                name,
                ipa_sudocmdgroup.get("member_sudocmd", []),
                sudocmd,
                client.sudocmdgroup_add_member_sudocmd,
                client.sudocmdgroup_remove_member_sudocmd,
            )
    else:
        if ipa_sudocmdgroup:
            changed = True
            if not module.check_mode:
                client.sudocmdgroup_del(name=name)

    return changed, client.sudocmdgroup_find(name=name)


def main():
    argument_spec = ipa_argument_spec()
    argument_spec.update(
        cn=dict(type="str", required=True, aliases=["name"]),
        description=dict(type="str"),
        state=dict(type="str", default="present", choices=["present", "absent", "enabled", "disabled"]),
        sudocmd=dict(type="list", elements="str"),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    client = SudoCmdGroupIPAClient(
        module=module,
        host=module.params["ipa_host"],
        port=module.params["ipa_port"],
        protocol=module.params["ipa_prot"],
    )
    try:
        client.login(username=module.params["ipa_user"], password=module.params["ipa_pass"])
        changed, sudocmdgroup = ensure(module, client)
        module.exit_json(changed=changed, sudorule=sudocmdgroup)
    except Exception as e:
        module.fail_json(msg=f"{e}", exception=traceback.format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for ios_hsrp_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: ios_hsrp_interfaces
short_description: Resource module to configure HSRP on interfaces.
description:
  - >-
    This module provides declarative management of HSRP configuration on
    interface for Cisco IOS devices.
version_added: 10.1.0
author:
  - Sagar Paul (@KB-perByte)
  - Nikhil Bhasin (@nickbhasin)
notes:
  - Tested against Cisco IOSXE Version 17.16.
  - >-
    This module works with connection C(network_cli). See
    U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
  - >-
    The module examples uses callback plugin (callback_result_format=yaml) to
    generate task output in yaml format.
  - >-
    For idempotency, the module consieders that version defaults to 1 as it is implied
    by the applaince and not available in the running-config.
    Priority defaults to 100 if not specified in the configuration.
options:
  config:
    description: A list of HSP configuration options to add to interface
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - >-
            Full name of the interface excluding any logical unit number, i.e.
            GigabitEthernet0/1.
        type: str
        required: true
      bfd:
        description: Enable HSRP BFD
        type: bool
      delay:
        description: HSRP initialization delay
        type: dict
        suboptions:
          minimum:
            description: Delay at least this long
            type: int
          reload:
            description: Delay after reload
            type: int
      redirect:
        description: Redirect configuration
        type: dict
        suboptions:
          advertisement:
            description: >-
              Redirect advertisement messages (standby redirect advertisement
              authentication md5)
            type: dict
            suboptions:
              authentication:
                description: Authentication configuration
                type: dict
                suboptions:
                  key_chain:
                    description: Set key chain
                    type: str
                  key_string:
                    description: Set key string
                    type: str
                  encryption:
                    description: Set encryption 0 (unencrypted/default) or 7 (hidden)
                    type: str
                  time_out:
                    description: Set timeout
                    type: int
                  password_text:
                    description: Password text valid for plain text and and key-string
                    type: str
          timers:
            description: Adjust redirect timers
            type: dict
            suboptions:
              adv_timer:
                description: Passive router advertisement interval in seconds
                type: int
              holddown_timer:
                description: Passive router holddown interval in seconds
                type: int
      mac_refresh:
        description: >-
          Refresh MAC cache on switch by periodically sending packet from
          virtual mac address
        type: int
      use_bia:
        description: >-
          HSRP uses interface's burned in address (does not work with mac
          address)
        type: dict
        suboptions:
          scope:
            description: >-
              Use-bia applies to all groups on this interface or
              sub-interface
            type: bool
          set:
            description: Set use-bia
            type: bool
      version:
        description: HSRP version (Defaults to 1 when standby_options is present)
        type: int
        choices:
          - 1
          - 2
      standby_options:
        description: Group number and group options for standby (HSRP)
        type: list
        elements: dict
        aliases:
          - standby_groups
        suboptions:
          group_no:
            description: Group number
            type: int
          follow:
            description: Enable HSRP BFD
            type: str
          ip:
            description: Enable HSRP IPv4 and set the virtual IP address
            type: list
            elements: dict
            suboptions:
              virtual_ip:
                description: Virtual IP address
                type: str
              secondary:
                description: Make this IP address a secondary virtual IP address
                type: bool
          ipv6:
            description: Enable HSRP IPv6 and set the IP address
            type: dict
            suboptions:
              autoconfig:
                description: Obtain address using autoconfiguration
                type: bool
              addresses:
                description: IPv6 link-local address or IPv6 prefix
                type: list
                elements: str
          mac_address:
            description: Virtual MAC address
            type: str
          group_name:
            description: Redundancy name string
            type: str
          authentication:
            description: Authentication configuration
            type: dict
            suboptions:
              key_chain:
                description: Set key chain
                type: str
              key_string:
                description: Set key string
                type: str
              encryption:
                description: Set encryption 0 (unencrypted/default) or 7 (hidden)
                type: int
              time_out:
                description: Set timeout
                type: int
              password_text:
                description: Password text valid for plain text and and key-string
                type: str
          preempt:
            description: Overthrow lower priority Active routers
            type: dict
            suboptions:
              enabled:
                description: Enables preempt, drives the lone `standby <grp_no> preempt` command
                type: bool
              minimum:
                description: Delay at least this long
                type: int
              reload:
                description: Delay after reload
                type: int
              sync:
                description: Wait for IP redundancy clients
                type: int
              delay:
                description: Wait before preempting
                type: bool
          priority:
            description: Priority level (defaults to 100)
            type: int
            default: 100
          timers:
            description: Overthrow lower priority Active routers
            type: dict
            suboptions:
              hello_interval:
                description: Hello interval in seconds
                type: int
              hold_time:
                description: Hold time in seconds
                type: int
              msec:
                description: Specify hello interval in milliseconds
                type: dict
                suboptions:
                  hello_interval:
                    description: <15-999>  Hello interval in milliseconds
                    type: int
                  hold_time:
                    description: <60-3000>  Hold time in milliseconds
                    type: int
          track:
            description: Priority tracking
            type: list
            elements: dict
            suboptions:
              track_no:
                description: Track object number
                type: int
              decrement:
                description: Priority decrement
                type: int
              shutdown:
                description: Shutdown Group
                type: bool
  running_config:
    description:
      - This option is used only with state I(parsed).
      - >-
        The value of this option should be the output received from the IOS
        device by executing the command B(show running-config | section
        ^interface).
      - >-
        The state I(parsed) reads the configuration from C(running_config)
        option and transforms it into Ansible structured data as per the
        resource module's argspec and the value is then returned in the
        I(parsed) key within the result.
    type: str
  state:
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - parsed
    default: merged
    description:
      - The state the configuration should be left in
      - >-
        The states I(rendered), I(gathered) and I(parsed) does not perform any
        change on the device.
      - >-
        The state I(rendered) will transform the configuration in C(config)
        option to platform specific CLI commands which will be returned in the
        I(rendered) key within the result. For state I(rendered) active
        connection to remote host is not required.
      - >-
        The state I(gathered) will fetch the running configuration from device
        and transform it into structured data in the format as per the resource
        module argspec and the value is returned in the I(gathered) key within
        the result.
      - >-
        The state I(parsed) reads the configuration from C(running_config)
        option and transforms it into JSON format as per the resource module
        parameters and the value is returned in the I(parsed) key within the
        result. The value of C(running_config) option should be the same format
        as the output of command I(show running-config | section ^interface)
        executed on device. For state I(parsed) active connection to remote host
        is not required.
    type: str
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# Router#show running-config | section ^interface
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  no ip address
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  no ip address
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address


- name: Populate the device with HSRP interface configuration
  cisco.ios.hsrp_interfaces:
    state: merged
    config:
      - delay:
          minimum: 5555
          reload: 556
        mac_refresh: 45
        name: Vlan70
        redirect:
          advertisement:
            authentication:
              key_chain: HSRP_CHAIN
          timers:
            adv_timer: 10
            holddown_timer: 55
        standby_options:
          - authentication:
              encryption: 7
              key_string: 0123456789ABCDEF
            follow: MASTER_GROUP
            group_name: PRIMARY_GROUP
            group_no: 10
            ip:
              - secondary: true
                virtual_ip: 10.0.10.2
            preempt:
              delay: true
              enabled: true
              minimum: 100
              reload: 50
              sync: 30
            priority: 110
            timers:
              hold_time: 250
              msec:
                hello_interval: 200
            track:
              - decrement: 20
                track_no: 1
          - follow: MASTER_GROUP
            group_name: IPV6_GROUP
            group_no: 20
            ipv6:
              addresses:
                - '2001:db8:20::1/64'
              autoconfig: true
            mac_address: 0000.0000.0014
            priority: 120
        version: 2
      - delay:
          minimum: 100
          reload: 200
        name: Vlan100
        standby_options:
          - authentication:
              password_text: hello_secret
            group_name: BACKUP_GROUP
            group_no: 5
            ip:
              - virtual_ip: 192.168.1.1
            preempt:
              enabled: true
            priority: 150
            timers:
              hello_interval: 5
              hold_time: 15
            track:
              - decrement: 30
                track_no: 10
        version: 2
      - name: GigabitEthernet3
        standby_options:
          - group_no: 1
            ip:
              - virtual_ip: 172.16.1.1
            priority: 100
        version: 1
      - name: GigabitEthernet2
        standby_options:
          - authentication:
              key_chain: AUTH_CHAIN
            group_no: 2
            ip:
              - secondary: true
                virtual_ip: 172.16.2.1
            priority: 100
        version: 1


# Task Output
# -----------
#
# before:
# -   name: GigabitEthernet1
# -   name: GigabitEthernet2
# -   name: GigabitEthernet3
# -   name: GigabitEthernet4
# -   name: Vlan70
# -   name: Vlan100
# commands:
# - interface Vlan70
# - standby version 2
# - standby delay minimum 5555 reload 556
# - standby mac-refresh 45
# - standby redirect timers 10 55
# - standby redirect advertisement authentication md5 key-chain HSRP_CHAIN
# - standby 10 follow MASTER_GROUP
# - standby 10 name PRIMARY_GROUP
# - standby 10 preempt delay minimum 100 reload 50 sync 30
# - standby 10 priority 110
# - standby 10 authentication md5 key-string 7 0123456789ABCDEF
# - standby 10 ip 10.0.10.2 secondary
# - standby 10 track 1 decrement 20
# - standby 20 follow MASTER_GROUP
# - standby 20 mac-address 0000.0000.0014
# - standby 20 name IPV6_GROUP
# - standby 20 priority 120
# - standby 20 ipv6 autoconfig
# - standby 20 ipv6 2001:db8:20::1/64
# - interface Vlan100
# - standby version 2
# - standby delay minimum 100 reload 200
# - standby 5 name BACKUP_GROUP
# - standby 5 preempt
# - standby 5 priority 150
# - standby 5 authentication ********
# - standby 5 ip 192.168.1.1
# - standby 5 track 10 decrement 30
# - interface GigabitEthernet3
# - standby version 1
# - standby 1 priority 100
# - standby 1 ip 172.16.1.1
# - interface GigabitEthernet2
# - standby version 1
# - standby 2 priority 100
# - standby 2 authentication md5 key-chain AUTH_CHAIN
# - standby 2 ip 172.16.2.1 secondary
# after:
# -   name: GigabitEthernet1
# -   name: GigabitEthernet2
#     standby_options:
#     -   authentication:
#             key_chain: AUTH_CHAIN
#         group_no: 2
#         ip:
#         -   secondary: true
#             virtual_ip: 172.16.2.1
#         priority: 100
# -   name: GigabitEthernet3
#     standby_options:
#     -   group_no: 1
#         ip:
#         -   virtual_ip: 172.16.1.1
#         priority: 100
# -   name: GigabitEthernet4
# -   delay:
#         minimum: 5555
#         reload: 556
#     mac_refresh: 45
#     name: Vlan70
#     redirect:
#         advertisement:
#             authentication:
#                 key_chain: HSRP_CHAIN
#         timers:
#             adv_timer: 10
#             holddown_timer: 55
#     standby_options:
#     -   authentication:
#             encryption: 7
#             key_string: 0123456789ABCDEF
#         follow: MASTER_GROUP
#         group_name: PRIMARY_GROUP
#         group_no: 10
#         ip:
#         -   secondary: true
#             virtual_ip: 10.0.10.2
#         preempt:
#             delay: true
#             enabled: true
#             minimum: 100
#             reload: 50
#             sync: 30
#         priority: 110
#         track:
#         -   decrement: 20
#             track_no: 1
#     -   follow: MASTER_GROUP
#         group_name: IPV6_GROUP
#         group_no: 20
#         ipv6:
#             addresses:
#             - 2001:DB8:20::1/64
#             autoconfig: true
#         mac_address: 0000.0000.0014
#         priority: 120
#     version: 2
# -   delay:
#         minimum: 100
#         reload: 200
#     name: Vlan100
#     standby_options:
#     -   group_name: BACKUP_GROUP
#         group_no: 5
#         ip:
#         -   virtual_ip: 192.168.1.1
#         preempt:
#             enabled: true
#         priority: 150
#         track:
#         -   decrement: 30
#             track_no: 10
#     version: 2

# After state:
# ------------
#
# Router#show running-config | section ^interface
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# !
# interface GigabitEthernet2
#  no ip address
#  standby 2 ip 172.16.2.1 secondary
#  standby 2 authentication md5 key-chain AUTH_CHAIN
#  shutdown
#  negotiation auto
# !
# interface GigabitEthernet3
#  no ip address
#  standby 1 ip 172.16.1.1
#  shutdown
#  negotiation auto
# !
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# !
# interface Vlan70
#  description for test
#  no ip address
#  standby mac-refresh 45
#  standby redirect timers 10 55
#  standby delay minimum 5555 reload 556
#  standby redirect advertisement authentication md5 key-chain HSRP_CHAIN
#  standby version 2
#  standby 10 ip 10.0.10.2 secondary
#  standby 10 follow MASTER_GROUP
#  standby 10 priority 110
#  standby 10 preempt delay minimum 100 reload 50 sync 30
#  standby 10 authentication md5 key-string 7 0123456789ABCDEF
#  standby 10 name PRIMARY_GROUP
#  standby 10 track 1 decrement 20
#  standby 20 ipv6 autoconfig
#  standby 20 ipv6 2001:DB8:20::1/64
#  standby 20 follow MASTER_GROUP
#  standby 20 priority 120
#  standby 20 name IPV6_GROUP
#  standby 20 mac-address 0000.0000.0014
# !
# interface Vlan100
#  description for test
#  no ip address
#  standby delay minimum 100 reload 200
#  standby version 2
#  standby 5 ip 192.168.1.1
#  standby 5 priority 150
#  standby 5 preempt
#  standby 5 name BACKUP_GROUP
#  standby 5 track 10 decrement 30
# !


# Using replaced

# Before state:
# -------------
#
# Router#show running-config | section ^interface
# interface Loopback888
#  no ip address
# interface Loopback999
#  no ip address
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  no ip address
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  no ip address
#  speed 1000
#  standby 22 ip 10.0.0.1 secondary
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  standby 0 priority 5
#  shutdown
#  negotiation auto

- name: Replaces device configuration of listed interfaces with provided configuration
  cisco.ios.ios_hsrp_interfaces:
    config:
      - name: GigabitEthernet3
        standby_groups:
          - group_no: 22
            ip:
              - virtual_ip: 10.0.0.1
                secondary: true
      - name: GigabitEthernet4
        standby_groups:
          - group_no: 0
            priority: 6
    state: replaced

# Task Output
# -----------
#
# before:
# - name: GigabitEthernet1
# - name: GigabitEthernet2
# - name: GigabitEthernet3
#     standby_groups:
#       - group_no: 22
#         ip:
#           - virtual_ip: 192.168.0.2
#             secondary: True
# - name: GigabitEthernet4
#     standby_groups:
#       - group_no: 0
#         priority: 6
# - name: Loopback999
# - name: Loopback888
# commands:
# - interface GigabitEthernet3
# - standby 22 ip 10.0.0.1 secondary
# - interface GigabitEthernet4
# - standby 0 priority 5
# after:
#   name: GigabitEthernet1
#   name: GigabitEthernet2
#   name: GigabitEthernet3
#     standby_groups:
#       - group_no: 22
#         ip:
#           - virtual_ip: 192.168.0.2
#             secondary: True
# - name: GigabitEthernet4
#     standby_groups:
#       - group_no: 0
#         priority: 6
# - name: Loopback999
# - name: Loopback888

# After state:
# ------------
#
# interface Loopback888
#  no ip address
# interface Loopback999
#  no ip address
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  no ip address
#  ip ospf network broadcast
#  ip ospf resync-timeout 10
#  ip ospf dead-interval 5
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  no ip address
#  standby 22 ip 10.0.0.1 secondary
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  description Auto_Cable_Testing_Ansible
#  no ip address
#  standby 0 priority 6
#  shutdown
#  negotiation auto

# Using overridden

# Before state:
# -------------
#
# interface Loopback888
#  no ip address
# interface Loopback999
#  no ip address
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  no ip address
#  ip ospf network broadcast
#  ip ospf resync-timeout 10
#  ip ospf dead-interval 5
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  no ip address
#  standby 22 ip 10.0.0.1 secondary
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  description Auto_Cable_Testing_Ansible
#  no ip address
#  standby 0 priority 6
#  shutdown
#  negotiation auto

- name: Override device configuration of all interfaces with provided configuration
  cisco.ios.ios_hsrp_interfaces:
    config:
      - name: GigabitEthernet4
        standby_groups:
          - group_no: 0
            priority: 10
    state: overridden

# Task Output
# -----------
# before:
# - name: GigabitEthernet1
# - name: GigabitEthernet2
# - name: GigabitEthernet3
#     standby_groups:
#       - group_no: 22
#         ip:
#           - virtual_ip: 192.168.0.2
#             secondary: True
# - name: GigabitEthernet4
#     standby_groups:
#       - group_no: 0
#         priority: 6
# - name: Loopback999
# - name: Loopback888
# commands:
# - interface GigabitEthernet3
# - no standby 22 ip 10.0.0.1 secondary
# - interface GigabitEthernet4
# - no standby 0 priority 6
# - standby 0 priority 10
# after:
# - name: GigabitEthernet1
# - name: GigabitEthernet2
# - name: GigabitEthernet3
# - name: GigabitEthernet4
#     standby_groups:
#       - group_no: 0
#         priority: 10
# - name: Loopback999
# - name: Loopback888

# After state:
# ------------
#
# router-ios#show running-config | section ^interface
# interface Loopback888
#  no ip address
# interface Loopback999
#  no ip address
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  no ip address
#  ip ospf network broadcast
#  ip ospf resync-timeout 10
#  ip ospf dead-interval 5
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  no ip address
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  description Auto_Cable_Testing_Ansible
#  no ip address
#  standby 0 priority 10
#  shutdown
#  negotiation auto

# Using deleted

# Before state:
# -------------
#
# router-ios#show running-config | section ^interface
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# !
# interface GigabitEthernet2
#  no ip address
#  standby 2 ip 172.16.2.1 secondary
#  standby 2 authentication md5 key-chain AUTH_CHAIN
#  shutdown
#  negotiation auto
# !
# interface GigabitEthernet3
#  no ip address
#  standby 1 ip 172.16.1.1
#  shutdown
#  negotiation auto
# !
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# !
# interface Vlan70
#  description for test
#  no ip address
#  standby mac-refresh 45
#  standby redirect timers 10 55
#  standby delay minimum 5555 reload 556
#  standby redirect advertisement authentication md5 key-chain HSRP_CHAIN
#  standby version 2
#  standby 10 ip 10.0.10.2 secondary
#  standby 10 follow MASTER_GROUP
#  standby 10 priority 110
#  standby 10 preempt delay minimum 100 reload 50 sync 30
#  standby 10 authentication md5 key-string 7 0123456789ABCDEF
#  standby 10 name PRIMARY_GROUP
#  standby 10 track 1 decrement 20
#  standby 20 ipv6 autoconfig
#  standby 20 ipv6 2001:DB8:20::1/64
#  standby 20 follow MASTER_GROUP
#  standby 20 priority 120
#  standby 20 name IPV6_GROUP
#  standby 20 mac-address 0000.0000.0014
# !
# interface Vlan100
#  description for test
#  no ip address
#  standby delay minimum 100 reload 200
#  standby version 2
#  standby 5 ip 192.168.1.1
#  standby 5 priority 150
#  standby 5 preempt
#  standby 5 name BACKUP_GROUP
#  standby 5 track 10 decrement 30


- name: "Delete attributes of given interfaces (NOTE: This won't delete the interfaces)"
  cisco.ios.ios_hsrp_interfaces:
    state: deleted

# Task Output
# -----------
#
# before:
# -   name: GigabitEthernet1
# -   name: GigabitEthernet2
#     standby_options:
#     -   authentication:
#             key_chain: AUTH_CHAIN
#         group_no: 2
#         ip:
#         -   secondary: true
#             virtual_ip: 172.16.2.1
#         priority: 100
# -   name: GigabitEthernet3
#     standby_options:
#     -   group_no: 1
#         ip:
#         -   virtual_ip: 172.16.1.1
#         priority: 100
# -   name: GigabitEthernet4
# -   delay:
#         minimum: 5555
#         reload: 556
#     mac_refresh: 45
#     name: Vlan70
#     redirect:
#         advertisement:
#             authentication:
#                 key_chain: HSRP_CHAIN
#         timers:
#             adv_timer: 10
#             holddown_timer: 55
#     standby_options:
#     -   authentication:
#             encryption: 7
#             key_string: 0123456789ABCDEF
#         follow: MASTER_GROUP
#         group_name: PRIMARY_GROUP
#         group_no: 10
#         ip:
#         -   secondary: true
#             virtual_ip: 10.0.10.2
#         preempt:
#             delay: true
#             enabled: true
#             minimum: 100
#             reload: 50
#             sync: 30
#         priority: 110
#         track:
#         -   decrement: 20
#             track_no: 1
#     -   follow: MASTER_GROUP
#         group_name: IPV6_GROUP
#         group_no: 20
#         ipv6:
#             addresses:
#             - 2001:DB8:20::1/64
#             autoconfig: true
#         mac_address: 0000.0000.0014
#         priority: 120
#     version: 2
# -   delay:
#         minimum: 100
#         reload: 200
#     name: Vlan100
#     standby_options:
#     -   group_name: BACKUP_GROUP
#         group_no: 5
#         ip:
#         -   virtual_ip: 192.168.1.1
#         preempt:
#             enabled: true
#         priority: 150
#         track:
#         -   decrement: 30
#             track_no: 10
#     version: 2
# commands:
# - interface GigabitEthernet2
# - no standby version 1
# - no standby 2
# - interface GigabitEthernet3
# - no standby version 1
# - no standby 1
# - interface Vlan70
# - no standby version 2
# - no standby delay minimum 5555 reload 556
# - no standby mac-refresh 45
# - no standby redirect timers 10 55
# - no standby redirect advertisement authentication md5 key-chain HSRP_CHAIN
# - no standby 10
# - no standby 20
# - no standby version 2
# - interface Vlan100
# - no standby version 2
# - no standby delay minimum 100 reload 200
# - no standby 5
# - no standby version 2
# after:
# -   name: GigabitEthernet1
# -   name: GigabitEthernet2
# -   name: GigabitEthernet3
# -   name: GigabitEthernet4
# -   name: Vlan70
# -   name: Vlan100

# After state:
# -------------
#
# router-ios#show running-config | section ^interface
# !
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# !
# interface GigabitEthernet2
#  no ip address
#  shutdown
#  negotiation auto
# !
# interface GigabitEthernet3
#  no ip address
#  shutdown
#  negotiation auto
# !
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# !
# interface Vlan70
#  description for test
#  no ip address
# !
# interface Vlan100
#  description for test
#  no ip address
# !

# Using gathered

# Before state:
# -------------
# router-ios#show running-config | section ^interface
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# !
# interface GigabitEthernet2
#  no ip address
#  standby 2 ip 172.16.2.1 secondary
#  standby 2 authentication md5 key-chain AUTH_CHAIN
#  shutdown
#  negotiation auto
# !
# interface GigabitEthernet3
#  no ip address
#  standby 1 ip 172.16.1.1
#  shutdown
#  negotiation auto
# !
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# !
# interface Vlan70
#  description for test
#  no ip address
#  standby mac-refresh 45
#  standby redirect timers 10 55
#  standby delay minimum 5555 reload 556
#  standby redirect advertisement authentication md5 key-chain HSRP_CHAIN
#  standby version 2
#  standby 10 ip 10.0.10.2 secondary
#  standby 10 follow MASTER_GROUP
#  standby 10 priority 110
#  standby 10 preempt delay minimum 100 reload 50 sync 30
#  standby 10 authentication md5 key-string 7 0123456789ABCDEF
#  standby 10 name PRIMARY_GROUP
#  standby 10 track 1 decrement 20
#  standby 20 ipv6 autoconfig
#  standby 20 ipv6 2001:DB8:20::1/64
#  standby 20 follow MASTER_GROUP
#  standby 20 priority 120
#  standby 20 name IPV6_GROUP
#  standby 20 mac-address 0000.0000.0014
# !
# interface Vlan100
#  description for test
#  no ip address
#  standby delay minimum 100 reload 200
#  standby version 2
#  standby 5 ip 192.168.1.1
#  standby 5 priority 150
#  standby 5 preempt
#  standby 5 name BACKUP_GROUP
#  standby 5 track 10 decrement 30

- name: Gather facts for hsrp interfaces
  cisco.ios.ios_hsrp_interfaces:
    state: gathered

# Task Output
# -----------
#
# gathered:
# -   name: GigabitEthernet1
# -   name: GigabitEthernet2
#     standby_options:
#     -   authentication:
#             key_chain: AUTH_CHAIN
#         group_no: 2
#         ip:
#         -   secondary: true
#             virtual_ip: 172.16.2.1
#         priority: 100
# -   name: GigabitEthernet3
#     standby_options:
#     -   group_no: 1
#         ip:
#         -   virtual_ip: 172.16.1.1
#         priority: 100
# -   name: GigabitEthernet4
# -   delay:
#         minimum: 5555
#         reload: 556
#     mac_refresh: 45
#     name: Vlan70
#     redirect:
#         advertisement:
#             authentication:
#                 key_chain: HSRP_CHAIN
#         timers:
#             adv_timer: 10
#             holddown_timer: 55
#     standby_options:
#     -   authentication:
#             encryption: 7
#             key_string: 0123456789ABCDEF
#         follow: MASTER_GROUP
#         group_name: PRIMARY_GROUP
#         group_no: 10
#         ip:
#         -   secondary: true
#             virtual_ip: 10.0.10.2
#         preempt:
#             delay: true
#             enabled: true
#             minimum: 100
#             reload: 50
#             sync: 30
#         priority: 110
#         track:
#         -   decrement: 20
#             track_no: 1
#     -   follow: MASTER_GROUP
#         group_name: IPV6_GROUP
#         group_no: 20
#         ipv6:
#             addresses:
#             - 2001:DB8:20::1/64
#             autoconfig: true
#         mac_address: 0000.0000.0014
#         priority: 120
#     version: 2
# -   delay:
#         minimum: 100
#         reload: 200
#     name: Vlan100
#     standby_options:
#     -   group_name: BACKUP_GROUP
#         group_no: 5
#         ip:
#         -   virtual_ip: 192.168.1.1
#         preempt:
#             enabled: true
#         priority: 150
#         track:
#         -   decrement: 30
#             track_no: 10
#     version: 2

# Using rendered

- name: Render the commands for provided configuration
  cisco.ios.ios_hsrp_interfaces:
    config:
      - delay:
          minimum: 5555
          reload: 556
        mac_refresh: 45
        name: Vlan70
        redirect:
          advertisement:
            authentication:
              key_chain: HSRP_CHAIN
          timers:
            adv_timer: 10
            holddown_timer: 55
        standby_options:
          - authentication:
              encryption: 7
              key_string: 0123456789ABCDEF
            follow: MASTER_GROUP
            group_name: PRIMARY_GROUP
            group_no: 10
            ip:
              - virtual_ip: 10.0.10.1
              - secondary: true
                virtual_ip: 10.0.10.2
              - secondary: true
                virtual_ip: 10.0.10.3
            mac_address: 0000.0c07.ac0a
            preempt:
              delay: true
              enabled: true
              minimum: 100
              reload: 50
              sync: 30
            priority: 110
            timers:
              hold_time: 250
              msec:
                hello_interval: 200
            track:
              - decrement: 20
                track_no: 1
              - shutdown: true
                track_no: 2
          - follow: MASTER_GROUP
            group_name: IPV6_GROUP
            group_no: 20
            ipv6:
              addresses:
                - 2001:db8:10::1/64
                - 2001:db8:20::1/64
              autoconfig: true
            mac_address: 0000.0c07.ac14
            priority: 120
        version: 2
      - delay:
          minimum: 100
          reload: 200
        name: Vlan100
        standby_options:
          - authentication:
              password_text: VALUE_SPECIFIED_IN_NO_LOG_PARAMETER
            group_name: BACKUP_GROUP
            group_no: 5
            ip:
              - virtual_ip: 192.168.1.1
            preempt:
              enabled: true
            priority: 150
            timers:
              hello_interval: 5
              hold_time: 15
            track:
              - decrement: 30
                track_no: 10
        version: 2
      - name: GigabitEthernet3
        standby_options:
          - group_no: 1
            ip:
              - virtual_ip: 172.16.1.1
            priority: 100
        use_bia:
          set: true
      - name: GigabitEthernet2
        standby_options:
          - authentication:
              key_chain: AUTH_CHAIN
            group_no: 2
            ip:
              - secondary: true
                virtual_ip: 172.16.2.1
            priority: 100
    state: rendered

# Task Output
# -----------
#
# rendered:
# - interface Vlan70
# - standby version 2
# - standby delay minimum 5555 reload 556
# - standby mac-refresh 45
# - standby redirect timers 10 55
# - standby redirect advertisement authentication md5 key-chain HSRP_CHAIN
# - standby 10 follow MASTER_GROUP
# - standby 10 mac-address 0000.0c07.ac0a
# - standby 10 name PRIMARY_GROUP
# - standby 10 preempt delay minimum 100 reload 50 sync 30
# - standby 10 priority 110
# - standby 10 authentication md5 key-string 7 0123456789ABCDEF
# - standby 10 ip 10.0.10.1
# - standby 10 ip 10.0.10.2 secondary
# - standby 10 ip 10.0.10.3 secondary
# - standby 10 track 1 decrement 20
# - standby 10 track 2 shutdown
# - standby 20 follow MASTER_GROUP
# - standby 20 mac-address 0000.0c07.ac14
# - standby 20 name IPV6_GROUP
# - standby 20 priority 120
# - standby 20 ipv6 autoconfig
# - standby 20 ipv6 2001:db8:10::1/64
# - standby 20 ipv6 2001:db8:20::1/64
# - interface Vlan100
# - standby version 2
# - standby delay minimum 100 reload 200
# - standby 5 name BACKUP_GROUP
# - standby 5 preempt
# - standby 5 priority 150
# - standby 5 authentication ********
# - standby 5 ip 192.168.1.1
# - standby 5 track 10 decrement 30
# - interface GigabitEthernet3
# - standby version 1
# - standby use-bia scope interface
# - standby 1 priority 100
# - standby 1 ip 172.16.1.1
# - interface GigabitEthernet2
# - standby version 1
# - standby 2 priority 100
# - standby 2 authentication md5 key-chain AUTH_CHAIN
# - standby 2 ip 172.16.2.1 secondary

# Using parsed

# File: parsed.cfg
# ----------------
#
# interface Vlan70
#  no ip address
#  standby mac-refresh 45
#  standby redirect timers 10 55
#  standby delay minimum 5555 reload 556
#  standby redirect advertisement authentication md5 key-chain HSRP_CHAIN
#  standby version 2
#  standby 10 ip 10.0.10.1
#  standby 10 ip 10.0.10.2 secondary
#  standby 10 ip 10.0.10.3 secondary
#  standby 10 follow MASTER_GROUP
#  standby 10 timers msec 200 250
#  standby 10 priority 110
#  standby 10 preempt delay minimum 100 reload 50 sync 30
#  standby 10 authentication md5 key-string 7 0123456789ABCDEF
#  standby 10 name PRIMARY_GROUP
#  standby 10 mac-address 0000.0c07.ac0a
#  standby 10 track 1 decrement 20
#  standby 10 track 2 shutdown
#  standby 20 ipv6 2001:db8:10::1/64
#  standby 20 ipv6 2001:db8:20::1/64
#  standby 20 ipv6 autoconfig
#  standby 20 follow MASTER_GROUP
#  standby 20 priority 120
#  standby 20 name IPV6_GROUP
#  standby 20 mac-address 0000.0c07.ac14

# interface Vlan100
#  no ip address
#  standby bfd
#  standby delay minimum 100 reload 200
#  standby version 2
#  standby 5 ip 192.168.1.1
#  standby 5 timers 5 15
#  standby 5 priority 150
#  standby 5 preempt
#  standby 5 authentication hello_secret
#  standby 5 name BACKUP_GROUP
#  standby 5 track 10 decrement 30

# interface GigabitEthernet3
#  standby use-bia
#  standby 1 ip 172.16.1.1
#  standby 1 priority 100

# interface GigabitEthernet2
#  standby follow VLAN70_GROUP
#  standby 2 ip 172.16.2.1 secondary
#  standby 2 authentication md5 key-chain AUTH_CHAIN

# - name: Parse the provided configuration
#   cisco.ios.ios_hsrp_interfaces:
#     running_config: "{{ lookup('file', 'parsed.cfg') }}"
#     state: parsed

# Task Output
# -----------
#
# parsed:
# -   delay:
#         minimum: 5555
#         reload: 556
#     mac_refresh: 45
#     name: Vlan70
#     redirect:
#         advertisement:
#             authentication:
#                 key_chain: HSRP_CHAIN
#         timers:
#             adv_timer: 10
#             holddown_timer: 55
#     standby_options:
#     -   authentication:
#             encryption: 7
#             key_string: 0123456789ABCDEF
#         follow: MASTER_GROUP
#         group_name: PRIMARY_GROUP
#         group_no: 10
#         ip:
#         -   virtual_ip: 10.0.10.1
#         -   secondary: true
#             virtual_ip: 10.0.10.2
#         -   secondary: true
#             virtual_ip: 10.0.10.3
#         mac_address: 0000.0c07.ac0a
#         preempt:
#             delay: true
#             enabled: true
#             minimum: 100
#             reload: 50
#             sync: 30
#         priority: 110
#         timers:
#             hold_time: 250
#             msec:
#                 hello_interval: 200
#         track:
#         -   decrement: 20
#             track_no: 1
#         -   shutdown: true
#             track_no: 2
#     -   follow: MASTER_GROUP
#         group_name: IPV6_GROUP
#         group_no: 20
#         ipv6:
#             addresses:
#             - 2001:db8:10::1/64
#             - 2001:db8:20::1/64
#             autoconfig: true
#         mac_address: 0000.0c07.ac14
#         priority: 120
#     version: 2
# -   delay:
#         minimum: 100
#         reload: 200
#     name: Vlan100
#     standby_options:
#     -   authentication:
#             password_text: VALUE_SPECIFIED_IN_NO_LOG_PARAMETER
#         group_name: BACKUP_GROUP
#         group_no: 5
#         ip:
#         -   virtual_ip: 192.168.1.1
#         preempt:
#             enabled: true
#         priority: 150
#         timers:
#             hello_interval: 5
#             hold_time: 15
#         track:
#         -   decrement: 30
#             track_no: 10
#     version: 2
# -   name: GigabitEthernet3
#     standby_options:
#     -   group_no: 1
#         ip:
#         -   virtual_ip: 172.16.1.1
#         priority: 100
#     use_bia:
#         set: true
# -   name: GigabitEthernet2
#     standby_options:
#     -   authentication:
#             key_chain: AUTH_CHAIN
#         group_no: 2
#         ip:
#         -   secondary: true
#             virtual_ip: 172.16.2.1
#         priority: 100
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - standby 22 ip 10.0.0.1 secondary
    - standby 0 priority 5
    - standby mac-refresh 21
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - standby 22 ip 10.0.0.1 secondary
    - standby 0 priority 5
    - standby mac-refresh 21
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.hsrp_interfaces.hsrp_interfaces import (
    Hsrp_interfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.hsrp_interfaces.hsrp_interfaces import (
    Hsrp_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Hsrp_interfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Hsrp_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2018, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: github_webhook_info
short_description: Query information about GitHub webhooks
description:
  - Query information about GitHub webhooks.
requirements:
  - "PyGithub >= 1.3.5"
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
options:
  repository:
    description:
      - Full name of the repository to configure a hook for.
    type: str
    required: true
    aliases:
      - repo
  user:
    description:
      - User to authenticate to GitHub as.
    type: str
    required: true
  password:
    description:
      - Password to authenticate to GitHub with.
    type: str
  token:
    description:
      - Token to authenticate to GitHub with.
    type: str
  github_url:
    description:
      - Base URL of the GitHub API.
    type: str
    default: https://api.github.com

author:
  - "Chris St. Pierre (@stpierre)"
"""

EXAMPLES = r"""
- name: List hooks for a repository (password auth)
  community.general.github_webhook_info:
    repository: ansible/ansible
    user: "{{ github_user }}"
    password: "{{ github_password }}"
  register: ansible_webhooks

- name: List hooks for a repository on GitHub Enterprise (token auth)
  community.general.github_webhook_info:
    repository: myorg/myrepo
    user: "{{ github_user }}"
    token: "{{ github_user_api_token }}"
    github_url: https://github.example.com/api/v3/
  register: myrepo_webhooks
"""

RETURN = r"""
hooks:
  description: A list of hooks that exist for the repo.
  returned: always
  type: list
  elements: dict
  sample:
    - has_shared_secret: true
      url: https://jenkins.example.com/ghprbhook/
      events: [issue_comment, pull_request]
      insecure_ssl: "1"
      content_type: json
      active: true
      id: 6206
      last_response:
        status: active
        message: OK
        code: 200
"""

import traceback

GITHUB_IMP_ERR = None
try:
    import github

    HAS_GITHUB = True
except ImportError:
    GITHUB_IMP_ERR = traceback.format_exc()
    HAS_GITHUB = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib


def _munge_hook(hook_obj):
    retval = {
        "active": hook_obj.active,
        "events": hook_obj.events,
        "id": hook_obj.id,
        "url": hook_obj.url,
    }
    retval.update(hook_obj.config)
    retval["has_shared_secret"] = "secret" in retval
    if "secret" in retval:
        del retval["secret"]

    retval["last_response"] = hook_obj.last_response.raw_data
    return retval


def main():
    module = AnsibleModule(
        argument_spec=dict(
            repository=dict(type="str", required=True, aliases=["repo"]),
            user=dict(type="str", required=True),
            password=dict(type="str", no_log=True),
            token=dict(type="str", no_log=True),
            github_url=dict(type="str", default="https://api.github.com"),
        ),
        mutually_exclusive=(("password", "token"),),
        required_one_of=(("password", "token"),),
        supports_check_mode=True,
    )

    if not HAS_GITHUB:
        module.fail_json(msg=missing_required_lib("PyGithub"), exception=GITHUB_IMP_ERR)

    try:
        github_conn = github.Github(
            module.params["user"],
            module.params.get("password") or module.params.get("token"),
            base_url=module.params["github_url"],
        )
    except github.GithubException as err:
        module.fail_json(msg=f"Could not connect to GitHub at {module.params['github_url']}: {err}")

    try:
        repo = github_conn.get_repo(module.params["repository"])
    except github.BadCredentialsException as err:
        module.fail_json(msg=f"Could not authenticate to GitHub at {module.params['github_url']}: {err}")
    except github.UnknownObjectException as err:
        module.fail_json(
            msg=f"Could not find repository {module.params['repository']} in GitHub at {module.params['github_url']}: {err}"
        )
    except Exception as err:
        module.fail_json(
            msg=f"Could not fetch repository {module.params['repository']} from GitHub at {module.params['github_url']}: {err}",
            exception=traceback.format_exc(),
        )

    try:
        hooks = [_munge_hook(h) for h in repo.get_hooks()]
    except github.GithubException as err:
        module.fail_json(
            msg=f"Unable to get hooks from repository {module.params['repository']}: {err}",
            exception=traceback.format_exc(),
        )

    module.exit_json(changed=False, hooks=hooks)


if __name__ == "__main__":
    main()
